/*
 * LCD SETUP for EZX: Setup the LCD
 * Copyright (C) 2009  Meng Zhong Die Wu <497699123@qq.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "filebrowser.h"

//ezx
#include <ZGlobal.h>
#include <ezxres.h>
#include <qtimer.h> 
#include <qdatetime.h>

//qt
#include <qdir.h>

FileBrowser::FileBrowser(const QString &filter, QWidget *parent, const char *name, const int &dirfile)
        :QListBox(parent, name)
{
      
    fileName = "";
    nameFilter = filter;
    type = dirfile;
    
  
    connect(this, SIGNAL(highlighted(int)), this, SLOT(itemHighlighted(int)));
    connect(this, SIGNAL(selected(int)), this, SLOT(itemSelected(int)));
    connect(this, SIGNAL(clicked(QListBoxItem *)), this, SLOT(itemClicked(QListBoxItem *)));

//setSelectionMode(QListBox::Extended);
}
    
FileBrowser::~FileBrowser()
{
}
    
void FileBrowser::setDir(const QString &path)
{
    RES_ICON_Reader ir;
    
    QDir dir(path, nameFilter);    
    dir.setMatchAllDirs(true);
    
    dir.setFilter(QDir::Dirs | QDir::Hidden);
    
    if (!dir.isReadable())
        return;
    clear();

		QStringList entries = dir.entryList();
   if (!dir.isRoot()){
				QString st = "..";
				insertItem(ir.getIcon("FMMS_Personal_Folder_S.gif",1),st);
			}
    QStringList::ConstIterator it = entries.begin();
    while (it != entries.end()) {
        if ((*it != ".")&&(*it != "..")) {
            insertItem(ir.getIcon("FMMS_Personal_Folder_S.gif",1), *it);
        }
        ++it;
    }

    if(type == 1)
    {
    dir.setFilter(QDir::Files | QDir::Hidden);    
    entries = dir.entryList();
    it = entries.begin();
    while (it != entries.end()) {
        if (*it != ".") {
            insertItem(ir.getIcon("FMMS_Personal_File_S.gif",1), *it);
        }
        ++it;
    }
    }
	
	curPath = dir.absPath();
    basePath = dir.canonicalPath();
}
    
void FileBrowser::itemHighlighted(int index)
{
    QString path = basePath + "/" + text(index);
    if (QFileInfo(path).isFile()) {
        fileName = path;
        emit picked(path);
        emit isFilePicked(TRUE);
    } else {
        fileName = "";
        emit isFilePicked(FALSE);
    }
      
if (QFileInfo(path).isDir()/*QDir(path).dirName()!= ".."*/){
       DirPath =path;
    emit dirpicked(path);
    emit isDirPicked(TRUE);
   
    }else {
        DirPath = "";
        emit isDirPicked(FALSE);
    }

if (QDir(basePath).isRoot()){
       DirPath = basePath+ text(index);
    //emit dirpicked(DirPath);
    emit isDirPicked(TRUE);
    }

}

void FileBrowser::itemSelected(int index)
{
    QString path = basePath + "/" + text(index);
    if (QFileInfo(path).isDir())
       
      setDir(path);
    
    if (QFileInfo(path).isFile()) {
        fileName = path;
       
        emit isFileClicked(TRUE);
    } else {
       
        emit isFileClicked(FALSE);
    }
}
    
void FileBrowser::itemClicked(QListBoxItem *item)
{
   QString path = basePath + "/" + text(index(item));
   if (QFileInfo(path).isDir())
      setDir(path);
}
