/*************************************************************************
 * Install Dialog: A install dialog 
 * Copyright (C) 2009 Meng Zhong Die Wu <497699123@qq.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***********************************************************************/

#include "installdialog.h"
#include "filebrowser.h"
#include <qvbox.h>
#include <qhbox.h>
#include <UTIL_DlgCst.h>
#include <ZMultiLineEdit.h>
#include <ZPushButton.h>
#include <ZScrollView.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfile.h>
#include <qlabel.h>

InstallDialog::InstallDialog(const QString &Title,QWidget *parent, char const *name)
  : UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, parent,name,ZBaseDialog::APP_MODAL,0)
{
	setDlgTitle(Title);

	ZScrollView* sv = new ZScrollView(this);
	sv->enableClipper(true);
	sv->setHScrollBarMode( QScrollView::AlwaysOff );
	sv->setVScrollBarMode( QScrollView::AlwaysOff );
	sv->setFrameStyle( QFrame::NoFrame );

	setDlgContent(sv);

	QLabel* label_1 = new QLabel( tr("Name:") ,sv->viewport());
	label_1->resize(200,24);
	 sv->addChild(label_1 , 5, 25 );

	lineedit_1 = new ZMultiLineEdit( sv->viewport(),true,1);
	lineedit_1->resize(180, 25);
	lineedit_1->setReadOnly(true);
	sv->addChild(lineedit_1, 5, 50);

	ZPushButton *filebutton = new ZPushButton("FMMS_FolderTree",sv->viewport());
	filebutton->resize(24, 24);
	sv->addChild(filebutton, 180,50);
	QObject::connect( filebutton, SIGNAL(clicked()), this, SLOT(SelectFile()) );

	QLabel* label_2 = new QLabel( tr("Install at:") ,sv->viewport());
	label_2->resize(200,24);
	sv->addChild(label_2 , 5, 100 );

	lineedit_2 = new ZMultiLineEdit( sv->viewport(),true,1);
	lineedit_2->resize(180, 25);
	lineedit_2->setReadOnly(true);
	sv->addChild(lineedit_2, 5, 125);

	ZPushButton *dirbutton = new ZPushButton("FMMS_FolderTree",sv->viewport());
	dirbutton->resize(24, 24);
	sv->addChild(dirbutton, 180,125);
	QObject::connect( dirbutton, SIGNAL(clicked()), this, SLOT(SelectFolder()) );

	UTIL_DlgCST* dlgcst = new UTIL_DlgCST( this , UTIL_DlgCST::dctOKCancel);
	dlgcst->getBtn1()->setText(tr("OK"));
	dlgcst->getBtn2()->setText(tr("Cancel"));
	QObject::connect(dlgcst->getBtn1(), SIGNAL(clicked()), this, SLOT(accept()));
	QObject::connect(dlgcst->getBtn2(), SIGNAL(clicked()), this, SLOT(reject()));
	setDlgCst(dlgcst);
}
InstallDialog::~InstallDialog()
{
}
void InstallDialog::setFileName(const QString &str)
{
	filename = str;
	lineedit_1->setText(QFileInfo( str ).fileName());
}
void InstallDialog::setDirName(const QString &str)
{
	dirname = str;
	lineedit_2->setText(QDir(str).dirName());
}
QString InstallDialog::getFileName()
{
	return filename;
}
QString InstallDialog::getDirName()
{
    return dirname;
}
void InstallDialog::SelectFile()
{
    
    //QStringList filter = "*.mtf";
    //ZOpenDlg* sf = new ZOpenDlg( filter,"/mmc/mmca1",false);
	UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
	Dialog->setHasTitle(true);
	Dialog->setDlgTitle(tr("select a mtf file"));

	QVBox *cw = new QVBox(Dialog);
	Dialog->setDlgContent(cw);
	QVBox *vb = (QVBox*)Dialog->getDlgContent();

	FileBrowser *mBrowser = new FileBrowser("*.mtf", vb, "browser",1);
	mBrowser->setDir("/mmc/mmca1/");
	QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

	QHBox *hb = new QHBox(Dialog);
	Dialog->setDlgCst(hb);
	
	ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
	okBtn->setText(tr("OK"));	
	connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
	ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
	cancelBtn->setText(tr("Cancel"));	
	connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));

 
	Dialog->show();
	if ( Dialog->exec() == QDialog::Accepted) 
	{
		filename=mBrowser->getFileName();   //防止重新载入出错
		lineedit_1->setText(QFileInfo( filename ).fileName());
	}
	delete Dialog;
}
void InstallDialog::SelectFolder()
{
	UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
	Dialog->setHasTitle(true);
	Dialog->setDlgTitle(tr("Select a folder"));

	QVBox *cw = new QVBox(Dialog);
	Dialog->setDlgContent(cw);
	QVBox *vb = (QVBox*)Dialog->getDlgContent();

	FileBrowser *mBrowser = new FileBrowser("*", vb, "browser",0);
	mBrowser->setDir("/mmc/mmca1/");
	QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

	QHBox *hb = new QHBox(Dialog);
	Dialog->setDlgCst(hb);
	
	ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
	okBtn->setText(tr("OK"));	
	connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
	ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
	cancelBtn->setText(tr("Cancel"));	
	connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));

 
	Dialog->show();
	if ( Dialog->exec() == QDialog::Accepted) 
	{
		dirname= mBrowser->getDirPath();
		lineedit_2->setText(mBrowser->getDirPath());
	}
    /*if(sfd->exec() == QDialog::Accepted)
	  {
        dirname = sfd->getSelected(&dirname);
        lineedit_2->setText(QDir(dirname).dirName());
    }*/
	delete Dialog;
}
