/*
 * LCD SETUP for EZX: Setup the LCD
 * Copyright (C) 2009 nukin <wbsecg1@gmail.com>
                      Meng Zhong Die Wu <497699123@qq.com>
                      
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 Name : LCD SETUP
 By : Meng Zhong Die Wu, nukin
 Compile : MOTO-E6-IDE
 Time : 2009.07.25
 EMail : 497699123@qq.com
*/
#include "lcdinfo.h"
#if HAVE_MODULE(MODULE_LCD)
//#include "filecodec.cpp"
#include <qsplitter.h>
#include <ezxpm.h>
#include <ezxres.h>
#include <ezxutilpushbutton.h>
#include <UTIL_Slider.h>
#include <ZDateTimePicker.h>
#include <ZPushButton.h>
#include <ZScrollView.h>
#include <ZSetManager.h>

#include <qwidget.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qvbuttongroup.h> 
#include <qhbox.h>
#include <qradiobutton.h>
#include <qwhatsthis.h>
#include <qtextstream.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <wait.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/ioctl.h>

LcdInfo::LcdInfo( QWidget *parent , const char *name, WFlags f )
    : QWidget( parent, name, f )
{
	QVBoxLayout *vb0 = new QVBoxLayout( this );
	vb0->setAutoAdd( true );

	ZScrollView* sv = new ZScrollView(this);
	sv->enableClipper(true);
	sv->setResizePolicy( QScrollView::AutoOneFit );
   // sv->setHScrollBarMode( QScrollView::AlwaysOff );
    //sv->setFrameStyle( QFrame::NoFrame );
	
	QWidget *container=new QWidget(sv->viewport());
	sv->addChild(container,true);

	QLabel *dispLabel=new QLabel(QString("<h3>%1</h3>").arg(tr("Display off")),container);
	//neverBtn = new QRadioButton( tr("Never"), container, "never");
	//defaultBtn = new QRadioButton( tr("30s default"), container, "default");
	//cuzBtn=new QRadioButton(tr("Customized"),container);
	timeCombo=new QComboBox(false,container);
	timeCombo->insertItem(tr("Never"),0);
	timeCombo->insertItem(tr("30s default"),1);
	timeCombo->insertItem(tr("Customized"),2);
	
	connect(timeCombo,SIGNAL(activated(int)),SLOT(setLcdSleepTime(int)));
	
	cuzTimeSpin=new QSpinBox(container);
	cuzTimeSpin->setSuffix("s");

	int time=getLcdSleepTime();
	timeCombo->setCurrentItem(time==0 ? 0 : (time==30 ? 1 : 2));
	//neverBtn->setChecked(time==0);
	//defaultBtn->setChecked(time==30);
	
	DOTime = new QLabel(QString("<center><h2>%1s</h2></center>").arg( time==0 ? un_gbk("∞") : QString::number(time) ), container, NULL);


	QFrame *sp0 = new QFrame( container );
	sp0->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	
	label = new QLabel( QString("<h3>%1 %1%</h3>").arg(tr("Brightness:")).arg(getBklightBrightness()) ,container);

	//slider=new QSlider(0,100,10,50,Horizontal, container);
	RES_ICON_Reader ir;
	QPixmap dot=ir.getIcon("MediaEditor_Color_Picker_Blue_S_Dot",true);
	QPixmap dotd=ir.getIcon("MediaEditor_Color_Picker_Blue_S_Dot",true);
	QPixmap slot=ir.getIcon("MediaEditor_Color_Picker_S_Slot",true);
	QPixmap slotd=ir.getIcon("MediaEditor_Color_Picker_S_Slot_D",true);
	slider=new UTIL_Slider(dot,dotd,slot,slotd,1,1,5,getBklightBrightness(),Qt::Horizontal,container,0,100);
	slider->setFixedSize(QSize(slot.width(),dot.height()));
	slider->setTracking(true);
	slider->setMaxValue(100);
	slider->setValue(getBklightBrightness());
	slider->setTickmarks(QSlider::Below);
	connect(slider,SIGNAL( sliderMoved(int)),SLOT(setBklightBrightness(int)));
	connect(slider,SIGNAL( sliderReleased()),SLOT(setBklightBrightness()));
	//QVButtonGroup* sensorSetup = new QVButtonGroup(tr("Lightsensord"), container );
	QFrame *sp = new QFrame( container );
	sp->setFrameStyle( QFrame::HLine | QFrame::Sunken );

	QLabel *l=new QLabel(QString("<h3>%1</h3>").arg(tr("Lightsensord")),container);
	int minBt, maxBt, camBt, detectInt;
	SETUP_SET_Manager mgr( "/tmp/sensor.cfg", false);
	mgr.getSettingInfo("Lightsensord Settings", "Min brightness", minBt ,0);
	mgr.getSettingInfo("Lightsensord Settings", "Min brightness", maxBt ,99);
	mgr.getSettingInfo("Lightsensord Settings", "Min brightness", camBt ,50);
	mgr.getSettingInfo("Lightsensord Settings", "Min brightness", detectInt ,3);

	QLabel *minBtLabel = new QLabel( tr("Min brightness:"),container);
	minBtS = new QSpinBox( 0, 99 , 1,container);
	minBtS->setValue(minBt);
	//hb->setStretchFactor(minBtLabel,155);

	QLabel *maxBtLabel = new QLabel( tr("Max brightness:"),container);
	maxBtS = new QSpinBox( 0, 99, 1,container);
	maxBtS->setValue(maxBt);

	QLabel *cameraLabel = new QLabel( tr("Camera brightness"),container);
	cameraBtS = new QSpinBox( 0, 99, 1,container);
	cameraBtS->setValue(camBt);

	QLabel *detectIntLabel = new QLabel( tr("Detect interval"),container);
	detectIntS = new QSpinBox( 0, 99, 1,container);
	detectIntS->setValue(detectInt);

	switchBt = new UTIL_PushButton( container, -1, -1, -1 );
	switchBt->setFont( font() );
	connect(minBtS,SIGNAL(valueChanged(int)),this,SLOT(setLightSensord()));
	connect(maxBtS,SIGNAL(valueChanged(int)),this,SLOT(setLightSensord()));
	connect(cameraBtS,SIGNAL(valueChanged(int)),this,SLOT(setLightSensord()));
	connect(detectIntS,SIGNAL(valueChanged(int)),this,SLOT(setLightSensord()));
	connect( switchBt, SIGNAL(clicked()), this, SLOT(setLightSensord()) );

	switchBt->setText(get_lightsensord_pid() ? tr("Turn off") : tr("Turn on"));
	
	
	
	QBoxLayout *vb = new QBoxLayout(container,QBoxLayout::Down);
	
	QGridLayout *gl=new QGridLayout(vb,2,3);
	gl->addWidget(timeCombo,0,0);
	gl->addWidget(cuzTimeSpin,1,0);
	//gl->addWidget(neverBtn,0,0);
	//gl->addWidget(defaultBtn,1,0);
	gl->addMultiCellWidget(DOTime,0,1,1,2);
	//gl->addWidget(cuzBtn,2,0);

	
	QGridLayout *gl2=new QGridLayout(vb,5,2);
	int r=0;
	gl2->addWidget(l,r,0);
	gl2->addWidget(switchBt,r++,1);
	gl2->addWidget(minBtLabel,r,0);
	gl2->addWidget(minBtS,r++,1);
	gl2->addWidget(maxBtLabel,r,0);
	gl2->addWidget(maxBtS,r++,1);
	gl2->addWidget(cameraLabel,r,0);
	gl2->addWidget(cameraBtS,r++,1);
	gl2->addWidget(detectIntLabel,r,0);
	gl2->addWidget(detectIntS,r++,1);
	
	//vb->addSpacing(10);
	vb->addWidget(dispLabel);
	vb->addSpacing(10);
	vb->addLayout(gl);
	//vb->addWidget(cuzBtn);
	vb->addSpacing(10);
	vb->addWidget(sp0);
	//vb->addSpacing(5);
	vb->addWidget(label);
	//vb->addSpacing(5);
	vb->addWidget(slider);
	vb->addSpacing(10);
	vb->addWidget(sp);
	vb->addSpacing(10);
	//vb->addWidget(l);
	vb->addLayout(gl2);
	//vb->addWidget(switchBt);
	
	

	QWhatsThis::add( this, tr("Setup the LCD."));
}
LcdInfo::~LcdInfo()
{
}
void LcdInfo::setLightSensord()
{
	if(get_lightsensord_pid())
	{
		switchBt->setText(tr("Turn on"));
		system( "start-stop-daemon --stop -s KILL --name lightsensord");
	}
	else
	{
		switchBt->setText(tr("Turn off"));
		//export first;
		qDebug(getenv("PATH"));
		system( QString( "lightsensord -m %1 -x %2 -e %3 -i %4").arg(minBtS->text()).arg(maxBtS->text()).arg(cameraBtS->text()).arg(detectIntS->text()) );
	}
}

int LcdInfo::get_lightsensord_pid()
{
   int child_pid = 0;
   FILE* fd = popen("pidof lightsensord","r");
   if (fd)
   {
		fscanf(fd,"%d\n",&child_pid);
		//printf("get_lightsensord_pid = %d\n",child_pid);
		pclose( fd );
    }
   return child_pid;
}
void LcdInfo::setLcdSleepTime(int var)
{
	int time;
	if(var == 0)
	{
		time = 0;
		PM_setupLcdSleepTime(0);
		DOTime->setText(QString("<center><h2>%1s</h2></center>").arg(un_gbk("∞")));
	}
	else if(var == 1)
	{
		time = 30;
		PM_setupLcdSleepTime(30);
		DOTime->setText(QString("<center><h2>%1s</h2></center>").arg("30") );
	}
	else
	{
		QTime qT(0,0,0);
		//qT=qT.addSecs(getLcdSleepTime());
		//qDebug(qT.toString());
		ZTimePickerDialog *timeDlg=new ZTimePickerDialog(qT,ZTimePicker::minute,ZTimePicker::clock24);
		if(timeDlg->exec()==QDialog::Accepted)
		{
			time=qT.secsTo(timeDlg->getTime());
			PM_setupLcdSleepTime(time);
			DOTime->setText(QString("<center><h2>%1s</h2></center>").arg(time));
			cuzTimeSpin->setValue(time);
			//neverBtn->setChecked(false);
			//defaultBtn->setChecked(false);
		}
	}
	SETUP_SET_Manager mgr( "/ezx_user/download/appwrite/setup/ezx_system.cfg", false);
	mgr.setSettingInfo("SYS_SYSTEM_SET", "LcdSleepTime", time);
}
int LcdInfo::getLcdSleepTime()
{
	int SleepTime;
	SETUP_SET_Manager mgr( "/ezx_user/download/appwrite/setup/ezx_system.cfg", false);
	mgr.getSettingInfo("SYS_SYSTEM_SET", "LcdSleepTime", SleepTime ,30);
	return SleepTime;
}

void LcdInfo::setBklightBrightness()
{
	setBklightBrightness(slider->value());
}

void LcdInfo::setBklightBrightness(int val)
{
#ifndef USE_EZXPM
	int fd;
	if (( fd = open ( "/dev/fb0", O_RDWR )) >= 0 ) {
		if( ioctl( fd, FBIOSETBRIGHTNESS, val ) == -1 )
			perror( "ioctl(\"FBIOSETBRIGHTNESS\")" );
		  ::close ( fd );
	}
#else
	PM_setupBklightBrightness(val);
#endif //USE_EZXPM
	SETUP_SET_Manager mgr( "/ezx_user/download/appwrite/setup/ezx_system.cfg", false);
	mgr.setSettingInfo("SYS_SYSTEM_SET", "Brightness", val);
	label->setText( tr("Brightness: %1%").arg(val));
}
int LcdInfo::getBklightBrightness()
{
#ifndef USE_EZXPM
	int fd;
	unsigned int val;  //unsigned 
	if (( fd = open ( "/dev/fb0", O_RDWR )) >= 0 ) {
		if( ioctl( fd, FBIOGETBRIGHTNESS, &val ) == -1 )
			val = 0;
	  else
			val = val & 0x00FF;
		::close ( fd );
	}
    return val;
#else
	int BklightBrightness;
	SETUP_SET_Manager mgr( "/ezx_user/download/appwrite/setup/ezx_system.cfg", false);
	mgr.getSettingInfo("SYS_SYSTEM_SET", "Brightness", BklightBrightness ,25);
	return BklightBrightness;
#endif //USE_EZXPM
}
#endif //MODULE_LCD

