/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "graph.h"
#include "memory.h"
#include "storage.h"  //deviceTab()
#include "AM_RegistryManager.h"
#include <ZApplication.h>
#include <ZConfig.h>
#include <stdio.h>
#include <ZMessageBox.h>
#include <ZPushButton.h>
#include <ZScrollView.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qfontmetrics.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qfile.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <stdlib.h>
#include <ezxutildialog.h>

#include <unistd.h>

const QString uuidStr="{f4d9ff87-4b74-45d3-af09-0c0a05af4d8f}";

QString appDir() {
	QString procdir=QString("/proc/self/exe");//%1/exe").arg(getpid());
	qDebug("proc: "+procdir);
	char abspath[1024];
	readlink(procdir.latin1(),abspath,1023);
	return QFileInfo(abspath).dirPath(true);
}

MemoryInfo::MemoryInfo( QWidget *parent, const char *name, WFlags f )
    : QWidget( parent, name, f )
{
	
	group=uuidStr.mid(1,uuidStr.length()-2);
	useSwapFile=true;
    QVBoxLayout *vb = new QVBoxLayout( this, 1 );

    totalMem = new QLabel( this );
    vb->addWidget( totalMem );

    data = new GraphData();
    graph = new BarGraph( this );
    graph->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    vb->addWidget( graph, 1 );
    graph->setData( data );

    legend = new GraphLegend( this );
    vb->addWidget( legend );
    legend->setData( data );

	swapBtn = new ZPushButton(NULL,tr("Swap"),this);
	swapBtn->setFixedSize(230,20);
	vb->addWidget( swapBtn );
	connect(swapBtn,SIGNAL(clicked()),SLOT(swapOptions()));
	
    swapMem = new QLabel( this );
    vb->addWidget( swapMem );

    swapdata = new GraphData();
    swapgraph = new BarGraph( this );
    swapgraph->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    vb->addWidget( swapgraph, 1 );
    swapgraph->setData( swapdata );

    swaplegend = new GraphLegend( this );
    vb->addWidget( swaplegend );
    swaplegend->setData( swapdata );

    vb->addStretch( 1 );
    updateData();

    QTimer *t = new QTimer( this );
    connect( t, SIGNAL( timeout() ), this, SLOT( updateData() ) );
    t->start( 5000 );

	QWhatsThis::add( this, tr( "This page shows how memory (i.e. RAM) is being allocated on your device.\n1. Used - memory used to by running applications.\n2. Buffers - temporary storage used to improve performance\n3. Cached - information that has recently been used, but has not been freed yet.\n4. Free - memory not currently used.\n" ) );

}

MemoryInfo::~MemoryInfo()
{
    delete data;
}

void MemoryInfo::updateData()
{
    QFile file( "/proc/meminfo" );

    if ( file.open( IO_ReadOnly ) )
    {
        // local variables
        QString line;
        QString identifier;
        QString value;
        int position;
        QTextStream t( &file );

        while ( !t.atEnd() )
        {
            // read a line
            line = t.readLine();

            // extract identifier and value from line
            position = line.find( ":" );
            identifier = line.left( position );
            value = line.mid( position + 1, line.length() - position - 3 );
            value = value.stripWhiteSpace();

            // copy values in variables
            if ( identifier == "MemTotal" )
            {
                total = value.toULong();
            } else if ( identifier == "MemFree" )
            {
                memfree = value.toULong();
            } else if ( identifier == "Buffers" )
            {
                buffers = value.toULong();
            } else if ( identifier == "Cached" )
            {
                cached = value.toULong();
            } else if ( identifier == "SwapCached" )
            {
            } else if ( identifier == "SwapTotal" )
            {
                swaptotal = value.toULong();
            } else if ( identifier == "SwapFree" )
            {
                swapfree = value.toULong();
            }
        }

        file.close();

        // calculate values
        used = total - memfree;
        swapused = swaptotal - swapfree;
        realUsed = total - ( buffers + cached + memfree );

        // visualize values
        totalMem->setText( tr( "Total Memory: %1 kB" ).arg( total ) );
        data->clear();
        data->addItem( tr("Used (%1 kB)").arg(realUsed), realUsed );
        data->addItem( tr("Buffers (%1 kB)").arg(buffers), buffers );		//高速缓存
        data->addItem( tr("Cached (%1 kB)").arg(cached), cached );		//缓存
        data->addItem( tr("Free (%1 kB)").arg(memfree), memfree );

        graph->hide();
        graph->show();
        legend->update();

		if(!deviceTab("/dev/mmca"))
		{
			swapMem->setText(tr("No SD Card. No Swap."));
			swapMem->show();
			swapBtn->setEnabled(false);
			//return;	//Floating point exception		
        }

        if (swaptotal > 0)
        {
            swapMem->setText( tr( "Total Swap: %1 kB" ).arg( swaptotal ) );
            swapdata->clear();
            swapdata->addItem( tr("Used (%1 kB)").arg(swapused), swapused );
            swapdata->addItem( tr("Free (%1 kB)").arg(swapfree), swapfree );

            swapMem->show();
            swapgraph->show();
            swaplegend->show();

            swapgraph->repaint( FALSE );
            swaplegend->update();
        }else{
            swapMem->hide();
            swapgraph->hide();
            swaplegend->hide();
        }

    }
}

void MemoryInfo::swapOptions()
{
	if(!deviceTab("/dev/mmca")) return;

    QString onoff = swaptotal > 0 ? tr("Turn Off") : tr("Turn On");
	switch(ZMessageBox::information ( this, NULL,//QPixmap( QDir::currentDirPath() + "/icon-big.png" ),
      onoff + tr(" swap?"), onoff , tr("Setup"),tr("Cancel") ))
	{
		case 0:
        {
			if(swaptotal > 0)
			{
				QFile file( "/proc/swaps" );
				if ( file.open( IO_ReadOnly ) )
				{
					QTextStream t( &file );
					QString swapname;
					while ( !t.atEnd() )
					{
						swapname=t.readLine();
						if(swapname.find("/")==-1)	continue;
						swapname=swapname.left(swapname.find(" "));
						system(QString("start-stop-daemon -S -c root -x /sbin/swapoff -p 0777 -v ")+swapname);
					}
				}
			}else
			{
				if(useSwapFile) 		
				{
					if(system(QString("start-stop-daemon -S -c root -x /sbin/swapon -p 0777 -v %1").arg(SWAPFILE))!=0)
					system("start-stop-daemon -S -c root -x /sbin/swapon -p 0644 -v /dev/mmca?");
				}
				else  system("start-stop-daemon -S -c root -x /sbin/swapon -p 0644 -v /dev/mmca?");
			}
			updateData();
			break;
		}
        case 1:
		{
			UTIL_Dialog *swapSetDlg=new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true,this,"swap setup",ZBaseDialog::APP_MODAL,0);
			swapSetDlg->setDlgTitle(tr("Settings..."));
			//QVBoxLayout *vbl = new QVBoxLayout( swapSetDlg, 5 );
			QVBox *cw = new QVBox(swapSetDlg);
			swapSetDlg->setDlgContent(cw);
			QVBox *vb = (QVBox*)swapSetDlg->getDlgContent();
   
			int y = 5;
			ZScrollView* sv = new ZScrollView(vb);
			sv->enableClipper(true);

			QCheckBox *autoSwap = new QCheckBox(tr("Swap on when boot up"),sv->viewport());
			//QFontMetrics fm(font());
			autoSwap->setMinimumWidth(220);//fm.width(tr("Swap on when boot up"),QString(tr("Swap on when boot up")).length()));
			autoSwap->setChecked(isAutoSwap());
			connect(autoSwap,SIGNAL(toggled(bool)),SLOT(slotAutoSwap(bool)));
			sv->addChild(autoSwap,10,y);
			
			y+=30;
			//QCheckBox *swapSize = new QCheckBox(tr("Size"),swapSetDlg);
			QLabel *swapSize = new QLabel(tr("Size"),swapSetDlg);
			//swapSize->setText();
			//vbl->addWidget(swapSize);
			//swapSize->setChecked(false);
			swapSize->resize(60,25);
			sv->addChild(swapSize,10,y); 
						
			//QHBoxLayout *hbl=new QHBoxLayout(swapSetDlg);
			QSpinBox *swapSizeValue=new QSpinBox(sv->viewport());
			swapSizeValue->setButtonSymbols(QSpinBox::PlusMinus);
			//hbl->addWidget(swapSizeValue);
			swapSizeValue->resize(130,25);
			swapSizeValue->setEnabled(false);
			swapSizeValue->setSuffix("Mb");
			swapSizeValue->setMaxValue(1024);
			swapSizeValue->setValue(QFileInfo(SWAPFILE).size()>>20);
			sv->addChild(swapSizeValue,60,y); 
			
			y+=35;

			//vbl->addLayout(hbl);
			
			QButtonGroup *use=new QButtonGroup(2,Qt::Vertical,tr("Type"),sv->viewport());
			//hbl->addWidget(use);
			use->resize(200,80);
			QRadioButton *pat=new QRadioButton(tr("Use swap partition"),use,"swappartiton");
			QRadioButton *file=new QRadioButton(tr("Use swap file"),use,"swapfile");
			connect(file,SIGNAL(toggled(bool)),swapSizeValue,SLOT(setEnabled(bool)));
			file->setChecked(true);
			sv->addChild(use,10,y); 
			
			

			QHBox *hb = new QHBox(swapSetDlg);
			swapSetDlg->setDlgCst(hb);
	
			ZPushButton *okBtn = new ZPushButton(hb,0,-1,-1);
			okBtn->setText(tr("OK"));	
			QObject::connect(okBtn, SIGNAL(clicked()),swapSetDlg,SLOT(accept()));
	
			ZPushButton *cancelBtn = new ZPushButton(hb,0,-1,-1);
			cancelBtn->setText(tr("Cancel"));	
			QObject::connect(cancelBtn, SIGNAL(clicked()),swapSetDlg,SLOT(reject()));
			
			connect(swapSize,SIGNAL(toggled(bool)),swapSizeValue,SLOT(setEnabled(bool)));

			if(swapSetDlg->exec()==QDialog::Accepted)
			{
				if(file->isChecked())
				{

					qDebug(" %d, %d",QFileInfo(SWAPFILE).size()>>20,swapSizeValue->value());
					if((QFileInfo(SWAPFILE).size()>>20)!=swapSizeValue->value()) {
						switch(ZMessageBox::information(this, NULL,tr("Swap file's size has been changed. Recreate it now?") \
							,tr("OK"),tr("Cancel"))) {
							case 0:
								{
									ZMessageBox::timerInformation(0,NULL,tr("Creating a file, please wait..."),1);
									system(QString("start-stop-daemon -S -c root -x /sbin/swapoff -p 0777 -v %1").arg(SWAPFILE));
									QFile(SWAPFILE).remove();
									int n=swapSizeValue->value()<<10;
									qDebug(QString("dd if=/dev/zero of=%1 bs=1024 count=%2").arg(SWAPFILE).arg(n));
									if(system(QString("dd if=/dev/zero of=%1 bs=1024 count=%2").arg(SWAPFILE).arg(n))!=0)
									{	
										qDebug("creat swap file faild!");
										ZMessageBox::timerInformation(0,NULL,tr("Fail to create a swap file!"),1);
									} else {
										ZMessageBox::timerInformation(0,NULL,tr("mkswap now, please wait..."),1);
										if(system(QString("mkswap %1 &").arg(SWAPFILE))!=0)
											ZMessageBox::timerInformation(0,NULL,tr("Fail to mkswap!"),1);
									}
									break;
								}
							case 1:
								break;
								
						}
					} 
					useSwapFile=true;
				} else {
					useSwapFile=false;
				}
			}
			break;
			
		}
		case 2:
			return;
		break;
   }
}

void MemoryInfo::slotAutoSwap(bool a)
{
	if(a) {
		if(isAutoSwap()) return;
		ZConfig sysreg("/ezx_user/download/appwrite/am/InstalledDB",true);
		//QUuid uuid=AM_RegistryManager::createUuid();
		QUuid uuid(uuidStr);
		
		sysreg.writeEntry(group,"AniIcon","");
		sysreg.writeEntry(group,"AppID",uuidStr);
		sysreg.writeEntry(group,"Args","-swapon");
		sysreg.writeEntry(group,"Attribute",1);//type 1 syspre
		sysreg.writeEntry(group,"BigIcon","");
		sysreg.writeEntry(group,"Daemon",1);//type 1
		//ZMessageBox::information(this,NULL,QDir::currentDirPath()+"  "+appDir(),"OK");
		sysreg.writeEntry(group,"Directory",QDir::currentDirPath());//appDir()); //appDir() is error, why?
		sysreg.writeEntry(group,"Eraseable",1);
		sysreg.writeEntry(group,"Exec",QFileInfo(qApp->argv()[0]).fileName());//Exec
		sysreg.writeEntry(group,"FixedAllPosition",0);
		sysreg.writeEntry(group,"FixedPosition",0);
		sysreg.writeEntry(group,"GroupID","root");
		sysreg.writeEntry(group,"Icon","");
		sysreg.writeEntry(group,"InstalledDate","2010/5/3");
		sysreg.writeEntry(group,"JavaId","-1");
		sysreg.writeEntry(group,"MassStorageStatus","");
		sysreg.writeEntry(group,"MpkgFile","");
		sysreg.writeEntry(group,"Name","SysInfo-SwapOn");
		sysreg.writeEntry(group,"OsVersion","");//Type 2
		sysreg.writeEntry(group,"Share","");//Type 2
		sysreg.writeEntry(group,"Size",60);//Type 2
		sysreg.writeEntry(group,"Type",2);//Type 2
		sysreg.writeEntry(group,"UserID","root");	
		sysreg.writeEntry(group,"Visible",0);
	
		qDebug(uuidStr);
		AM_RegistryManager am(this);
		//am.initRegistry();ZDEBUG
		am.addAppToAutoRun(uuid);
	
	} else {
		if(!isAutoSwap()) return;	
		AM_RegistryManager am(this);
		//am.initRegistry();
		am.removeAppFromAutoRun(group);	
	}
}

bool MemoryInfo::isAutoSwap()
{	
	ZConfig cf("/ezx_user/download/appwrite/am/UserConfig");
	QStringList autolist=cf.readListEntry("AutoRun","Apps",QChar(';'));
	if(autolist.contains(group)) return true;
	
	return false;
}

