/**********************************************************************
** ModulesInfo
**
** Display Modules information
**
** Copyright (C) 2002, Michael Lauer
**                    mickey@tm.informatik.uni-frankfurt.de
**                    http://www.Vanille.de
**
** Based on ProcessInfo by Dan Williams <williamsdr@acm.org>
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**********************************************************************/

/*******************************************
nukin
2009
12.27: add popup menu "add/remove/detail"

********************************************/

#include "modulesinfo.h"
#include "filebrowser.h"
#include "detail.h"
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <ezxres.h>
#include <ezxutildialog.h>
#include <ZApplication.h>
#include <ZPushButton.h>
#include <ZMessageBox.h>

#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qtimer.h>
#include <qfile.h>
#include <qlistview.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qwhatsthis.h>

ModulesInfo::ModulesInfo( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    QGridLayout *layout = new QGridLayout( this );
    layout->setSpacing( 0 );
    layout->setMargin( 0 );

    moduleList = new QListView( this );
    int colnum = moduleList->addColumn(tr("Name"));//QString::fromUtf8("模块" ) );
    colnum = moduleList->addColumn(tr("Size"));//QString::fromUtf8("大小" ) );
    moduleList->setColumnAlignment( colnum, Qt::AlignRight );
    colnum = moduleList->addColumn( tr( "Use#" ) );
    moduleList->setColumnAlignment( colnum, Qt::AlignRight );
    colnum = moduleList->addColumn( tr( "Used by" ) );
    moduleList->setAllColumnsShowFocus( TRUE );
    layout->addMultiCellWidget( moduleList, 0, 0, 0, 1 );
    QWhatsThis::add( moduleList,  tr( "This is a list of all the kernel modules currently loaded on this handheld device.\nPress and hold on a module to chose more options: Add, Remove and Details."));
	
	// Test if we have /sbin/modinfo, and if so, allow module detail window
    if ( QFile::exists( "/sbin/modinfo" )  )
    {
        // QPEApplication::setStylusOperation( moduleList->viewport(), QPEApplication::RightOnHold );
        ZApplication::setMouseMode( moduleList->viewport(), 1 );
        connect( moduleList, SIGNAL( rightButtonPressed(QListViewItem*,const QPoint&,int) ),
			this, SLOT( optionsMenu(QListViewItem*, const QPoint & ) ) );
    }

    CommandCB = new QComboBox( FALSE, this );
    CommandCB->insertItem( "modprobe -r" );
    CommandCB->insertItem( "rmmod" );
    // I can't think of other useful commands yet. Anyone?
    layout->addWidget( CommandCB, 1, 0 );
    QWhatsThis::add( CommandCB,tr( "Select a command here and then click the Send button to the right to send the command to module selected above." ) );

    QPushButton *btn = new QPushButton( this );
    btn->setFixedWidth(60);
    btn->setText(tr("Send"));//QString::fromUtf8("发送" ) );
    connect( btn, SIGNAL( clicked() ), this, SLOT( slotSendClicked() ) );
    layout->addWidget( btn, 1, 1 );
    QWhatsThis::add( btn, tr( "Click here to send the selected command to the module selected above." ) );

    QTimer *t = new QTimer( this );
    connect( t, SIGNAL( timeout() ), this, SLOT( updateData() ) );
    t->start( 5000 );
	
    updateData();
	
	ModulesDtl = new Detail();
	QWhatsThis::add( ModulesDtl->detailView, tr( "This area shows detailed information about this module." ) );
}

ModulesInfo::~ModulesInfo()
{
}

void ModulesInfo::updateData()
{
    char modname[64];
    char usage[200];
    int modsize, usecount;

    QString selectedmod;
    QListViewItem *curritem = moduleList->currentItem();
    if ( curritem )
    {
        selectedmod = curritem->text( 0 );
    }
    
    moduleList->clear();

    FILE *procfile = fopen( ( QString ) ( "/proc/modules"), "r");

    if ( procfile )
    {
		QListViewItem *newitem;
		QListViewItem *selecteditem = 0x0;
		while ( true ) 
		{
			int success = fscanf( procfile, "%s%d%d%[^\n]", modname, &modsize, &usecount, usage );
			if ( success == EOF ) break;
			QString qmodname = QString( modname );
			QString qmodsize = QString::number( modsize ).rightJustify( 6, ' ' );
			QString qusecount = QString::number( usecount ).rightJustify( 2, ' ' );
			QString qusage = QString( usage );

			newitem = new QListViewItem( moduleList, qmodname, qmodsize, qusecount, qusage );
			if ( qmodname == selectedmod ) selecteditem = newitem;
		}
		moduleList->setCurrentItem( selecteditem );

		fclose( procfile );
	}
}

void ModulesInfo::slotSendClicked()
{
	if(!moduleList->currentItem())	return;
	QString modName = moduleList->currentItem()->text( 0 );
	QString command = CommandCB->currentText()+QString(" ")+modName;
	QString capstr = tr("You will run the command: ")+command;
	
	RES_ICON_Reader ir;
	switch(ZMessageBox::information ( this,ir.getIcon(RES_ICON_DLG_WAITING,1), capstr, "Sure"," Cancel"))
	{
		case 0:
		{
			FILE* stream = popen( command, "r" );
			if ( stream ) pclose( stream );
		}
		case 1:
        	break;        
		break;
	};
}

void ModulesInfo::optionsMenu(QListViewItem *modules, const QPoint & pos)
{
	QPopupMenu *options=new QPopupMenu(this,NULL);
	options->insertItem(tr("Add"),this,SLOT(addAModule()),0);
	options->insertItem(tr("Remove"),this,SLOT(removeThisModule()),0);
	options->insertSeparator();
	options->insertItem(tr("Details"),this,SLOT(moduleDetails()),0);	
	options->setItemEnabled(options->idAt(1),modules!=NULL);
	options->setItemEnabled(options->idAt(3),modules!=NULL);		
	options->popup(pos);
}

void ModulesInfo::addAModule()
{
	UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"select module",1,0);
	Dialog->setHasTitle(true);
	Dialog->setDlgTitle("select a module");

	QVBox *cw = new QVBox(Dialog);
	Dialog->setDlgContent(cw);
	QVBox *vb = (QVBox*)Dialog->getDlgContent();

	FileBrowser *moduleView=new FileBrowser("*.o",vb,0,1);
	moduleView->setDir("/lib/modules");
	
	QHBox *hb = new QHBox(Dialog);
	Dialog->setDlgCst(hb);
	
	ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
	okBtn->setText("OK");	
	connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
	ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
	cancelBtn->setText("Cancel");	
	connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));

    if(Dialog->exec() == QDialog::Accepted)
	{
		FILE* stream;
		if((stream=popen(QString("insmod ")+moduleView->getFileName(),"r"))!=NULL)
		{
			//how to show erro msg?
			pclose(stream);
        }
		//system(QString("insmod ")+moduleView->getFileName());
		updateData();
	}
	delete Dialog;
}
void ModulesInfo::removeThisModule()
{
	
	QString modName=moduleList->currentItem()->text( 0 );
	switch(ZMessageBox::information ( this, NULL, tr("Sure to remove this module: %1?").arg(modName),"Yes"," No"))
	{
        case 0:
        {
			QString command = "/sbin/rmmod ";
			command.append( modName );
			FILE* stream = popen( QString("/sbin/rmmod ")+modName, "r" );
			if( stream == NULL )
			{
				char buf[2000];
				sprintf(buf,"error: %s\n", strerror(errno));
				printf("%s\n",buf);
				ModulesDtl->detailView->append(buf);
				ModulesDtl->setGeometry(10,40,220,250);
				ModulesDtl->show();
        	}
        	else pclose(stream);
			updateData();
		}
        case 1:
			break;       
        break;
	};
}
void ModulesInfo::moduleDetails()
{
	//if(moduleList->currentItem()==NULL) qDebug("no item"); return;  //not work
    QString modname = moduleList->currentItem()->text( 0 ); //moduleSelected->text( 0 )
    QString capstr = "Module: ";
    capstr.append( modname );
    ModulesDtl->setCaption( capstr );
    QString command = "/sbin/modinfo ";
    command.append( "/lib/modules/" );
    command.append( modname );
    command.append( ".o" );
    FILE* modinfo = popen( command, "r" );
	
    if ( modinfo )
    {
        char line[200];
        ModulesDtl->detailView->setText( " Details:\n------------\n" );
        while( true )
        {
            int success = fscanf( modinfo, "%[^\n]\n", line );
            if ( success == EOF )
                break;
            ModulesDtl->detailView->append( line );
        }
        pclose( modinfo );
    }
    ModulesDtl->showMaximized();
}

