#include "processinfo.h"
#include "detail.h"

#include <ezxres.h>
#include <ZApplication.h>
#include <ZMessageBox.h>

#include <qpushbutton.h>
#include <qcombobox.h>
#include <qcopchannel_qws.h>
#include <qtimer.h>
#include <qdir.h>
#include <qlistview.h>
#include <qtextview.h>
#include <qpopupmenu.h>
#include <qwhatsthis.h>

#include <signal.h>
#include <unistd.h>  //sysconf()

Process::Process() { 
    clear();
}

Process::Process(int _pid)  {
	clear();
    pid = _pid;
//    parent_pid = _ppid;
//    parent = _parent;
}

void Process::clear() {
    pid = 0; 
//    parent_pid = 0; 
//    tracerpid = 0;
    userTime = 0;
    sysTime = 0;
    elapsedTimeMilliSeconds = 0;
    userUsage=0;
    sysUsage=0;
    totalUserUsage=0;
    totalSysUsage=0;
//    numChildren=0;
//    niceLevel=0;
    vmSize=0;
    vmRSS = 0;
    vmURSS = 0;
    status="Unknow";
    memPct=0;
    cpuPct=0;
}

void Process::setUserTime(qlonglong _userTime) {
    userTime = _userTime;
} 
void Process::setSysTime(qlonglong _sysTime) {
    sysTime = _sysTime;
}  
void Process::setUserUsage(int _userUsage) {
    if(userUsage == _userUsage) return;
    userUsage = _userUsage;
} 
void Process::setSysUsage(int _sysUsage) {
    if(sysUsage == _sysUsage) return;
    sysUsage = _sysUsage;
}  
void Process::setTotalUserUsage(int _totalUserUsage) {
    if(totalUserUsage == _totalUserUsage) return;
    totalUserUsage = _totalUserUsage;
} 
void Process::setTotalSysUsage(int _totalSysUsage) {
    if(totalSysUsage == _totalSysUsage) return;
    totalSysUsage = _totalSysUsage;
}
void Process::setVmSize(qlonglong _vmSize) {
    if(vmSize == _vmSize) return;
    vmSize = _vmSize;
}   
void Process::setVmRSS(qlonglong _vmRSS) {
    if(vmRSS == _vmRSS) return;
    vmRSS = _vmRSS;
}    
void Process::setVmURSS(qlonglong _vmURSS) {
    if(vmURSS == _vmURSS) return;
    vmURSS = _vmURSS;
}   
void Process::setName(const QString& _name) {
    if(name == _name) return;
    name = _name;
}
void Process::setCommand(const QString& _command) {
    if(command == _command) return;
    command = _command;
} 
void Process::setMemUsage(float _mem)
{
	memPct=_mem;
}
void Process::setCpuUsage(double _cpu)
{
	cpuPct=_cpu;
}
void Process::setStatus(const QString& st)
{
	status=st;
}




ProcessInfo::ProcessInfo( QWidget* parent,  const char* name, WFlags fl )
    :  QWidget( parent, name, fl )
{
    QGridLayout *layout = new QGridLayout( this );
    layout->setSpacing( 0 );
    layout->setMargin( 0 );
    ProcessView = new QListView( this, "ProcessView" );
    int colnum = ProcessView->addColumn( tr( "PID" ) );
    ProcessView->setColumnAlignment( colnum, Qt::AlignRight );
    colnum = ProcessView->addColumn( tr( "Command" ),96 );
    colnum = ProcessView->addColumn( tr( "CPU%" ) );
    colnum = ProcessView->addColumn( tr( "Statu" ) );
    colnum = ProcessView->addColumn( tr( "MEM%" ) );    
    colnum = ProcessView->addColumn( tr( "Time" ) );
    ProcessView->setColumnAlignment( colnum, Qt::AlignRight );
    ProcessView->setAllColumnsShowFocus( TRUE );
    ProcessView->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
   // QPEApplication::setStylusOperation( ProcessView->viewport(), QPEApplication::RightOnHold );

    ZApplication::setMouseMode( ProcessView->viewport(), 1 );
    connect( ProcessView,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint&,int)),SLOT(slotOptionsMenu(QListViewItem*,const QPoint &)));

    layout->addMultiCellWidget( ProcessView, 0, 0, 0, 1 );
    QWhatsThis::add( ProcessView, tr( "This is a list of all the processes on this handheld device.\nPress and hold on a process to see additional information about the process.\nnukin added cpu and memory usage." ) );
    
	SignalCB = new QComboBox( FALSE, this, "SignalCB" );
    SignalCB->insertItem( " 1: SIGHUP"  ,0);
    SignalCB->insertItem( " 2: SIGINT"  ,1);
    SignalCB->insertItem( " 3: SIGQUIT" ,2);
    SignalCB->insertItem( " 5: SIGTRAP" ,3);
    SignalCB->insertItem( " 6: SIGABRT" ,4);
    SignalCB->insertItem( " 9: SIGKILL" ,5);
    SignalCB->insertItem( "14: SIGALRM" ,6);
    SignalCB->insertItem( "15: SIGTERM" ,7);
    SignalCB->insertItem( "18: SIGCONT" ,8);
    SignalCB->insertItem( "19: SIGSTOP" ,9);
    SignalCB->setCurrentItem(5);

    layout->addWidget( SignalCB, 1, 0 );
    QWhatsThis::add( SignalCB, tr( "Select a signal here and then click the Send button to the right to send to this process.") );

    SendButton = new QPushButton( this, "SendButton" );
    SendButton->setFixedWidth(60);
    SendButton->setText( tr( "Send" ) );
    connect( SendButton, SIGNAL( clicked() ), this, SLOT( slotSendClicked() ) );
    layout->addWidget( SendButton, 1, 1 );
    QWhatsThis::add( SendButton, tr( "Click here to send the selected signal to this process." ) );
	
	gettimeofday( &last, 0 );
	getPidsInfo();
    updateData();
    ProcessDtl = new Detail();
    
    QTimer *t = new QTimer( this );
    connect( t, SIGNAL( timeout() ), this, SLOT( updateData() ) );
    t->start( 5000 );
    QWhatsThis::add( ProcessDtl->detailView,  tr( "This area shows detailed information about this process." ) );
    layout->addWidget( SendButton, 1, 1 );
    
	
	options=new QPopupMenu(this);
	options->insertItem(tr("Switch"),this,SLOT(switchApp()),0);
	options->insertSeparator();
	options->insertItem(tr("Details"),this,SLOT(processDetails()),0);
}

ProcessInfo::~ProcessInfo()
{
}

void ProcessInfo::slotOptionsMenu(QListViewItem *item, const QPoint & pos)
{
	options->setItemEnabled(options->idAt(0),item!=NULL);
	options->setItemEnabled(options->idAt(2),item!=NULL);		
	options->popup(pos);
	
}

void ProcessInfo::switchApp()
{
	QCopChannel::send(QCString("EZX/Application/"+ProcessView->currentItem()->text( 0 ).stripWhiteSpace()),QCString("raise()"),QByteArray());
}

void ProcessInfo::getPidsInfo()
{
	ProcessList tmpProcesses;
	int pid, ppid, pgrp, session, tty, tpgid, u_time, s_time, cutime, cstime, counter, priority, starttime,
       signal, blocked, sigignore, sigcatch;
    uint flags, minflt, cminflt, majflt, cmajflt, timeout, itrealvalue, vsize, rss, rlim, startcode,
         endcode, startstack, kstkesp, kstkeip, wchan;
    int mem_total, size, resident;//, share, trs, lrs, drs, dt ; //statm
    char state;
    char comm[64], buf[36], tmp[36];
    float testm=0.0;
    double testcpu=0.;
	struct timeval now;
    gettimeofday( &now, 0 );
    int tdiff = now.tv_usec - last.tv_usec;
    tdiff += (now.tv_sec - last.tv_sec) * 1000000;
    tdiff /= 10000; //10ms
	QDir *procdir = new QDir("/proc", 0, QDir::Name, QDir::Dirs);
    QFileInfoList *proclist = new QFileInfoList(*(procdir->entryInfoList()));
    if ( proclist )
    {
        QFileInfoListIterator it(*proclist);
        QFileInfo *f;
        while ( ( f = it.current() ) != 0 )
        {
            ++it; 
            QString pidNow=f->fileName();
            if ( pidNow >= "1" && pidNow <= "99999" )
            {
            	Process *ps=new Process(pidNow.toInt());
                FILE *procfile = fopen( ( QString ) ( "/proc/" + pidNow + "/stat"), "r");
                if ( procfile )
				{
					
                    fscanf( procfile,
                           "%d %s %c %d %d %d %d %d %u %u %u %u %u %d %d %d %d %d %d %u %u %d %u %u %u %u %u %u %u %u %d %d %d %d %u",
                           &pid, comm, &state, &ppid, &pgrp, &session,&tty, &tpgid, &flags, &minflt, &cminflt,
                           &majflt, &cmajflt, &u_time, &s_time, &cutime, &cstime, &counter, &priority, &timeout,
                           &itrealvalue, &starttime, &vsize, &rss, &rlim, &startcode, &endcode, &startstack,
                           &kstkesp, &kstkeip, &signal, &blocked, &sigignore, &sigcatch, &wchan );
                    Process *ps_last=processes.findPid(pidNow.toInt());
                    int diff=(u_time-ps_last->userTime)+(s_time-ps_last->sysTime);  //jiffies=10ms. pid changes
					double percent=(double)diff*100/(double)tdiff;
					testcpu+=percent;
					ps->setCpuUsage(percent);
					ps->setName(QString(comm).replace(QRegExp("[()]"),""));
					ps->setUserTime(u_time);
					ps->setSysTime(s_time);
					ps->setStatus(QChar(state));
                    fclose( procfile );
				}
				

				FILE *fp;
				//sprintf(fname, "/proc/%d/statm", pid);
				fp = fopen((QString)("/proc/"+pidNow+"/statm"),"r");
				fgets(buf, 36, fp);
				fclose(fp);
				//sscanf(buf, "%d %d %d %d %d %d %d", &size, &resident, &share, &trs, &lrs, &drs, &dt);
				sscanf(buf, "%d %d",  &size, &resident);
				fp = fopen("/proc/meminfo", "r");
				while(fgets(buf, 36, fp))
					if(strncmp(buf, "MemTotal", 8) == 0) sscanf(buf, "%s %d", &tmp, &mem_total);
				fclose(fp);
				//float mp=(resident<<2)/(float)mem_total*100.0f;  //Resident(pages) 应用程序正在使用的物理内存的大小 VmRSS/4,
				float mp=(resident*sysconf(_SC_PAGESIZE) / 1024)/(float)mem_total*100.0f;
				ps->setMemUsage(mp);
				testm+=mp;
				tmpProcesses.append(ps);
			}
		}
		processes=tmpProcesses;
		printf("mem sum: %.1f  cpu sum: %.1f\n",testm,testcpu);
	}
	delete proclist;
    delete procdir;
    last=now;
}

void ProcessInfo::updateData()
{
    QString selectedpid;
    QListViewItem *curritem = ProcessView->currentItem();
    if ( curritem )
    {
        selectedpid = curritem->text( 0 );
    }
    ProcessView->clear();

    QListViewItem *newitem;
    QListViewItem *selecteditem = 0x0;

    getPidsInfo();

    Process *ps;
    QListIterator<Process> it(processes);
	for(;it.current(); ++it) {
		ps=it.current();
    	newitem=new QListViewItem(ProcessView,QString::number(ps->pid).rightJustify(5,' '),ps->name,QString().sprintf("%.1f",ps->cpuPct).rightJustify(5,' '),ps->status,QString().sprintf("%.1f",ps->memPct).rightJustify(5,' '),QString::number((int)((ps->userTime+ps->sysTime)/100)).rightJustify(7,' '));
		if( ps->pid == selectedpid.toInt() ) selecteditem = newitem;
        ProcessView->setCurrentItem( selecteditem );
    }

}

void ProcessInfo::slotSendClicked()
{
	QListViewItem *currprocess = ProcessView->currentItem();
	if ( !currprocess ) return;
	
	QString capstr = tr( "Really want to send %1\nto this process [ %2 ]?" ).arg( SignalCB->currentText()).arg(currprocess->text(1));
	RES_ICON_Reader ir;
	switch(ZMessageBox::information ( this,ir.getIcon(RES_ICON_DLG_WAITING,1), capstr, "Yes"," No"))
	{
		case 0:
		{
			//currprocess = ProcessView->currentItem();
			if ( currprocess )
			{
				QString sigstr = SignalCB->currentText();
				sigstr.truncate(2);
				int sigid = sigstr.toUInt();
				kill( currprocess->text( 0 ).stripWhiteSpace().toUInt(), sigid );
				updateData();
			}
		}
        case 1:
			break;      
		break;
	}
}

void ProcessInfo::processDetails()
{
    QString pid= ProcessView->currentItem()->text( 0 ).stripWhiteSpace();
    QString command = ProcessView->currentItem()->text( 1 );
    ProcessDtl->setCaption( pid + " - " + command );
    FILE *statfile = fopen( ( QString ) ( "/proc/" + pid + "/status"), "r");
    if ( statfile )
    {
        char line[81];
        fgets( line, 81, statfile );
        ProcessDtl->detailView->setText( line );
        while ( fgets( line, 81, statfile ) )
        {
            ProcessDtl->detailView->append( line );
        }
        fclose( statfile );
    }
    ProcessDtl->showMaximized();
}

