#ifndef PROCESSINFO_H
#define PROCESSINFO_H

#include <qwidget.h>

#include <sys/time.h>
#define P_MAX 200
#include <qlist.h>


typedef unsigned long long int  qlonglong;
class Process
{
public:
	Process(int pid);
	Process();
	
	void clear();
	
	void setUserTime(qlonglong userTime); ///< The time, in 100ths of a second, spent in total on user calls. -1 if not known
    void setSysTime(qlonglong sysTime);  ///< The time, in 100ths of a second, spent in total on system calls.  -1 if not known
    void setUserUsage(int userUsage); ///< Percentage (0 to 100).  It might be more than 100% on multiple cpu core systems
    void setSysUsage(int sysUsage);  ///< Percentage (0 to 100).  It might be more than 100% on multiple cpu core systems
    void setTotalUserUsage(int totalUserUsage); ///< Percentage (0 to 100) from the sum of itself and all its children recursively.  If there's no children, it's equal to userUsage.  It might be more than 100% on multiple cpu core systems
    void setTotalSysUsage(int totalSysUsage); ///< Percentage (0 to 100) from the sum of itself and all its children recursively. If there's no children, it's equal to sysUsage. It might be more than 100% on multiple cpu core systems
	void setVmSize(qlonglong vmSize);   ///< Virtual memory size in KiloBytes, including memory used, mmap'ed files, graphics memory etc,
    void setVmRSS(qlonglong vmRSS);    ///< Physical memory used by the process and its shared libraries.  If the process and libraries are swapped to disk, this could be as low as 0
    void setVmURSS(qlonglong vmURSS);   ///< Physical memory used only by the process, and not counting the code for shared libraries. Set to -1 if unknown
    void setMemUsage(float mem);
    void setCpuUsage(double cpu);
    void setName(const QString& name);  ///< The name (e.g. "ksysguard", "konversation", "init")
    void setCommand(const QString& command); ///< The command the process was launched with
    void setStatus(const QString&); ///< Whether the process is running/sleeping/etc
    
//private:
	int pid;
    qlonglong userTime;
    qlonglong sysTime;
    int userUsage;
    int sysUsage;
    int totalUserUsage;
    int totalSysUsage;
	qlonglong vmSize;
    qlonglong vmRSS;
    qlonglong vmURSS;
    QString name;
    QString command;
    QString status;
    int elapsedTimeMilliSeconds;
    float memPct;
    double cpuPct;

};

class ProcessList :public QList<Process>
{
public:
	Process *findPid(int _pid) {
		Process *ps=new Process(_pid);
		QListIterator<Process> it(*this);
		for(;it.current(); ++it) {//ps=first();ps!=last();next()) {
			ps=it.current();
			if(ps->pid==_pid) break;
		}
		return ps;
	}

};


class QListView;
class QListViewItem;
class QComboBox;
class QPushButton;
class Detail;
class QPopupMenu;

class ProcessInfo : public QWidget
{
    Q_OBJECT
public:
    ProcessInfo( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    ~ProcessInfo();
private slots:
    void updateData();
    void slotSendClicked();
    void slotOptionsMenu(QListViewItem *item,const QPoint & pos);
    void switchApp();

    void processDetails();
private:
    QListView*   ProcessView;
    QComboBox*   SignalCB;
    QPushButton* SendButton;
    QPopupMenu *options;
   	Detail *ProcessDtl;
    struct timeval last;
    void getPidsInfo(); 
    ProcessList processes;
};

#endif
