/*
 * SysInfo: System Information
 * Copyright (C) 2009  Meng Zhong Die Wu <497699123@qq.com>
 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**********************************************************************
*****setup font
**********************************************************************
**
**  Enhancements by: nukin <wbsecg1@gmail.com>
**  
**********************************************************************/

#include "sysinfo.h"
#include "memory.h"
#include "load.h"
#include "storageinfo.h"
#include "processinfo.h"
#include "modulesinfo.h"
#include "versioninfo.h"
#include "sysloginfo.h"
#include "usbmode.h"
#include "themeinfo.h"
#include "lcdinfo.h"
#include "benchmarkinfo.h"
#include "detail.h"
//#include "about.cpp"

//#include "qtabwidget.h"

#include <ZSetManager.h>
#include <ZLanguage.h>
#include <ZPushButton.h>
#include <qfont.h>
#include <qfile.h>
#include <qdir.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qapplication.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <qwhatsthis.h>
#include <qlabel.h>
#include <qtimer.h>

#include <ezxutilpushbutton.h>
#include <ezxutilcst.h>

#include <stdlib.h>

SystemInfo::SystemInfo( QWidget *parent, const char *name, unsigned int WFlags )
    : ZMainWidget("",true,parent, name, WFlags/*WStyle_ContextHelp*/ )
{
	QFont fnt( font() );
	fnt.setPointSize(15);
	setFont( fnt );
	((QLabel *)getTitleBarWidget())->setText(tr("System Information"));

	wtButton = new UTIL_PushButton( getTitleBarWidget() , -1, -1, -1 );
	wtButton->setNormalOverlay( QPixmap( QDir::currentDirPath() + "/pics/whatsthis.png" ) );
	wtButton->setActiveOverlay( QPixmap( QDir::currentDirPath() + "/pics/whatsthis.png" ) );
	wtButton->resize( 25, 25 );
	wtButton->move( ZGlobal::getScreenR().width() - 25, 0 );
	connect( wtButton, SIGNAL(clicked()), this, SLOT(enterWhatsThisMode()) );
	resizeButton=new ZPushButton( "Dialog_Up_Arr",getTitleBarWidget() );
	resizeButton->setFixedSize(QSize(32,24));
	resizeButton->move(ZGlobal::getScreenR().width()-57,0);
	connect(resizeButton,SIGNAL(clicked()),this,SLOT(resizeWindow()));
    
	wtTimer = new QTimer( this );
	connect( wtTimer, SIGNAL(timeout()), this, SLOT(whatsThisTimeout()) );

	QVBox * vb = new QVBox(this);
	setContentWidget(vb);
	cw = (QVBox*)getContentWidget(0);
	//QVBoxLayout *lay = new QVBoxLayout( this );
	//QTabWidget *tab = new QTabWidget( this );
	//	qDebug("debug1");
	//lay->addWidget( tab );
	//tab->addTab( new MemoryInfo( tab ), tr("Memory") );
	//	qDebug("debug2");
	//tab->addTab( new LoadInfo( tab ), tr("CPU") );
	//memory = new MemoryInfo(cw);
	loadinfo=new LoadInfo(cw);
	//loadinfo->hide();
 
	cst = new UTIL_CST(this, tr("CPU"));
	setCSTWidget(cst);
	connect( cst->getRightBtn(), SIGNAL(clicked()),this, SLOT( close() ));
	//connect( cst->getMidBtn(), SIGNAL(clicked()),this, SLOT( setLcdinfo() ));
	
	midBtn=cst->getMidBtn();
	leftBtn = cst->getLeftBtn();
	mPopup = new QPopupMenu(cst, NULL);
	mPopup->setFont( fnt );
#if HAVE_MODULE(MODULE_MEMINFO)
	mPopup->insertItem(tr("Memory"), MEMINFO_IDX,MEMINFO_IDX);
#endif
#if HAVE_MODULE(MODULE_STORAGE)
	qDebug("storage idx %d",STORAGE_IDX);
	mPopup->insertItem(tr("Storage"), STORAGE_IDX, STORAGE_IDX);
#endif
#if HAVE_MODULE(MODULE_MODULES)
	mPopup->insertItem(tr("Modules"), MODULES_IDX,MODULES_IDX);
#endif
#if HAVE_MODULE(MODULE_PROCESS)
	mPopup->insertItem(tr("Process"),PROCESS_IDX,PROCESS_IDX);
#endif
#if HAVE_MODULE(MODULE_LOAD)
	mPopup->insertItem(tr("CPU"),CPU_IDX,CPU_IDX);
#endif
#if HAVE_MODULE(MODULE_USB)
	mPopup->insertItem(tr("USB Mode"),USB_IDX,USB_IDX);
#endif
#if HAVE_MODULE(MODULE_SYSLOG)
	mPopup->insertItem(tr("System Log"),SYSLOG_IDX,SYSLOG_IDX);
#endif
#if HAVE_MODULE(MODULE_BENCHMARK)
	mPopup->insertItem(tr("Benchmark"),BENCHMARK_IDX,BENCHMARK_IDX);
#endif
#if HAVE_MODULE(MODULE_THEME)
	mPopup->insertItem(tr("Themes"),THEME_IDX,THEME_IDX);
#endif
#if HAVE_MODULE(MODULE_LCD)
	mPopup->insertItem(tr("LCD"),LCD_IDX,LCD_IDX);
#endif
#if HAVE_MODULE(MODULE_VERSION)
	mPopup->insertItem(tr("Version"),VERSION_IDX,VERSION_IDX);
#endif
	QObject::connect(mPopup, SIGNAL(activated(int)), SLOT(selectMenu(int)));
	mPopup->setItemChecked(CPU_IDX,true); 	
	midBtn->setPopup(mPopup);
	
	lPopup=new QPopupMenu(cst, NULL);
	lPopup->setFont( fnt );
	lPopup->insertItem(tr("About"),this,SLOT(about()));
	lPopup->insertItem(tr("showFull"),this,SLOT(showFull()));
	leftBtn->setPopup(lPopup);
}
SystemInfo::~SystemInfo()
{
}

void SystemInfo::resizeWindow()
{
	if(rect()!=ZGlobal::getScreenR())
	{
		showMaximized();
		resizeButton->setResourceID("Dialog_Down_Arr");
	}
	else
	{
		showNormal();
		resizeButton->setResourceID("Dialog_Up_Arr");
	}
}

void SystemInfo::showFull()
{
	if(getFullScreenMode()) setFullScreenMode(false,false);
	else setFullScreenMode(true,true);
}

void SystemInfo::selectMenu(int idx)
{
	if(mPopup->isItemChecked(idx)) return;
	if( mPopup->isItemChecked(MEMINFO_IDX))
	{
#if HAVE_MODULE(MODULE_MEMINFO)
		delete  memory;
		mPopup->setItemChecked(MEMINFO_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_STORAGE)
	else if( mPopup->isItemChecked(STORAGE_IDX))
	{
		qDebug("storage idx %d",STORAGE_IDX);
		delete  filesysinfo;
		mPopup->setItemChecked(STORAGE_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_MODULES)
	else if( mPopup->isItemChecked(MODULES_IDX))
	{
		delete  modulesinfo;
		mPopup->setItemChecked(MODULES_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_PROCESS)
	else if( mPopup->isItemChecked(PROCESS_IDX))
	{
		delete processinfo;
		mPopup->setItemChecked(PROCESS_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_LOAD)
	else if( mPopup->isItemChecked(CPU_IDX))
	{
		loadinfo->hide();
		mPopup->setItemChecked(CPU_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_USB)
	else if( mPopup->isItemChecked(USB_IDX))
	{
		delete usbmode;
		mPopup->setItemChecked(USB_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_SYSLOG)
	else if( mPopup->isItemChecked(SYSLOG_IDX))
	{
		delete sysloginfo;
		mPopup->setItemChecked(SYSLOG_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_BENCHMARK)
	else if( mPopup->isItemChecked(BENCHMARK_IDX))
	{
		delete benchmarkinfo;
		mPopup->setItemChecked(BENCHMARK_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_THEME)
	else if( mPopup->isItemChecked(THEME_IDX))
	{
		delete themeinfo;
		mPopup->setItemChecked(THEME_IDX,false);

	}
#endif
#if HAVE_MODULE(MODULE_LCD)
	else if( mPopup->isItemChecked(LCD_IDX))
	{
		delete lcdinfo;
		mPopup->setItemChecked(LCD_IDX,false);
	}
#endif
#if HAVE_MODULE(MODULE_VERSION)	
	else if( mPopup->isItemChecked(VERSION_IDX))
	{
		delete versioninfo;
		mPopup->setItemChecked(VERSION_IDX,false);
	}
#endif
 /* for(int i=1; i<= 11;i++){
    mPopup->setItemChecked(i,i==idx); 
  }*/
  
	if(idx == MEMINFO_IDX){
#if HAVE_MODULE(MODULE_MEMINFO)
		memory = new MemoryInfo(cw);
		memory->show();
		midBtn->setText(tr("Memory"));
	}
#endif
#if HAVE_MODULE(MODULE_STORAGE)	
	else if(idx == STORAGE_IDX){
		filesysinfo=new FileSysInfo(cw);
		filesysinfo->show();
		midBtn->setText(tr("Storage"));
	}
#endif
#if HAVE_MODULE(MODULE_MODULES)	
	else if(idx == MODULES_IDX){
		modulesinfo=new ModulesInfo(cw);
		modulesinfo->show();
		midBtn->setText(tr("Modules"));
	}
#endif
#if HAVE_MODULE(MODULE_PROCESS)	
	else if(idx == PROCESS_IDX){
		processinfo=new ProcessInfo(cw);
		processinfo->show();
		midBtn->setText(tr("Process"));
	}
#endif
#if HAVE_MODULE(MODULE_LOAD)	
	else if(idx == CPU_IDX){
   // loadinfo=new LoadInfo(cw);
		loadinfo->show();
		midBtn->setText(tr("CPU")); 
	}
#endif
#if HAVE_MODULE(MODULE_USB)	
	else if(idx == USB_IDX){
		usbmode = new USBMode(cw);
		usbmode->show();
		midBtn->setText(tr("USB Mode"));
	}
#endif
#if HAVE_MODULE(MODULE_SYSLOG)	
	else if(idx == SYSLOG_IDX){
		sysloginfo = new SyslogInfo(cw);
		sysloginfo->show();
		midBtn->setText(tr("System Log"));
	}
#endif
#if HAVE_MODULE(MODULE_BENCHMARK)
	else if(idx == BENCHMARK_IDX){
		benchmarkinfo=new BenchmarkInfo(cw);
		benchmarkinfo->show();
		midBtn->setText(tr("Benchmark")); 
	}
#endif
#if HAVE_MODULE(MODULE_THEME)	
	else if(idx == THEME_IDX){
		themeinfo=new ThemeInfo(cw);
		themeinfo->show();
		midBtn->setText(tr("Themes")); 
	}
#endif
#if HAVE_MODULE(MODULE_LCD)	
	else if(idx == LCD_IDX){
		lcdinfo = new LcdInfo(cw);
		lcdinfo->show();
		midBtn->setText(tr("LCD"));

	}
#endif
#if HAVE_MODULE(MODULE_VERSION)
	else if(idx == VERSION_IDX){
		versioninfo=new VersionInfo(cw);
		versioninfo->show();
		midBtn->setText(tr("Version"));
	}
#endif
	mPopup->setItemChecked(idx,true); 
} 

void SystemInfo::enterWhatsThisMode()
{
    QWhatsThis::enterWhatsThisMode();
    if ( QWhatsThis::inWhatsThisMode() ) {
        wtButton->setDown( TRUE );
        ((QLabel *)getTitleBarWidget())->setText( tr("What's this?") );
        wtTimer->start(250);
    }
}
void SystemInfo::whatsThisTimeout()
{
    if ( !QWhatsThis::inWhatsThisMode() ) {
        wtButton->setDown( FALSE );
        ((QLabel *)getTitleBarWidget())->setText( tr("System Infomation") );
        wtTimer->stop();
    }
}

void SystemInfo::keyPressEvent ( QKeyEvent * event )
{

#define E6_KEY_VALUE_OK            Qt::Key_Return
#define E6_KEY_VALUE_LEFT          Qt::Key_Left
#define E6_KEY_VALUE_UP            Qt::Key_Up
#define E6_KEY_VALUE_RIGHT         Qt::Key_Right
#define E6_KEY_VALUE_DOWN          Qt::Key_Down
#define E6_KEY_VALUE_CALL          Qt::Key_F1
#define E6_KEY_VALUE_HANG          Qt::Key_F2
#define E6_KEY_VALUE_BROWSER       Qt::Key_F4
#define E6_KEY_VALUE_CAM           Qt::Key_F5
#define E6_KEY_VALUE_SHORTCUT      Qt::Key_F30
#define E6_KEY_VALUE_VOICE_UP      Qt::Key_PageUp
#define E6_KEY_VALUE_VOICE_DOWN    Qt::Key_PageDown
#define E6_KEY_VALUE_PLAY          Qt::Key_F26
#define E6_KEY_VALUE_PLAY_PRE      Qt::Key_F28
#define E6_KEY_VALUE_PLAY_NEXT     Qt::Key_F29

    switch (event->key())
    {
		//case E6_KEY_VALUE_BROWSER:
		//	setFullScreenMode(false,false);
		//	break;
		case E6_KEY_VALUE_SHORTCUT:
			showFull();
			event->accept();
			break;
        default:
			event->ignore();//break;
			break;
    }
    
}

void SystemInfo::about()
{
	QString file=QDir::currentDirPath()+"/about_"+ZLanguage::getSystemLanguageCode()+".qht";
	if(!QFile(file).exists()) file=QDir::currentDirPath()+"/about_"+ZLanguage::getSystemLanguageCode()+".html";
	if(!QFile(file).exists()) file=QDir::currentDirPath()+"/about_en-us.qht";
	if(!QFile(file).exists()) file=QDir::currentDirPath()+"/about_en-us.html";
	if(!QFile(file).exists()) return;
	QFile aboutFile(file);
	//qDebug(QDir::currentDirPath()+"/about_en-us.qht");
	if(!aboutFile.open(IO_ReadOnly)) qDebug("open error!");
	QTextStream stream(&aboutFile);
	stream.setCodec(QTextCodec::codecForName("utf8"));
	Detail *aboutDlg=new Detail;	
	aboutDlg->detailView->setText(stream.read());
	aboutFile.close();

    aboutDlg->showMaximized();
}

