/*
 * SysInfo: System Information
 * Copyright (C) 2009  Meng Zhong Die Wu <497699123@qq.com>
 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef SYSINFO_H
#define SYSINFO_H
#include "config.h"

#define MEMINFO_IDX		(HAVE_MODULE(MODULE_MEMINFO)	? 0 				: -1)
#define STORAGE_IDX		(HAVE_MODULE(MODULE_STORAGE)	? MEMINFO_IDX+1 	: MEMINFO_IDX)
#define MODULES_IDX		(HAVE_MODULE(MODULE_MODULES) 	? STORAGE_IDX+1 	: STORAGE_IDX)
#define PROCESS_IDX		(HAVE_MODULE(MODULE_PROCESS) 	? MODULES_IDX+1 	: MODULES_IDX)
#define CPU_IDX			(HAVE_MODULE(MODULE_LOAD)		? PROCESS_IDX+1 	: PROCESS_IDX)
#define USB_IDX			(HAVE_MODULE(MODULE_USB) 		? CPU_IDX+1 		: CPU_IDX)
#define SYSLOG_IDX		(HAVE_MODULE(MODULE_SYSLOG) 	? USB_IDX+1 		: USB_IDX)
#define BENCHMARK_IDX	(HAVE_MODULE(MODULE_BENCHMARK) 	? SYSLOG_IDX+1 		: SYSLOG_IDX)
#define THEME_IDX		(HAVE_MODULE(MODULE_THEME)		? BENCHMARK_IDX+1 	: BENCHMARK_IDX)
#define LCD_IDX			(HAVE_MODULE(MODULE_LCD) 		? THEME_IDX+1 		: THEME_IDX)
#define VERSION_IDX		(HAVE_MODULE(MODULE_VERSION) 	? LCD_IDX+1 		: LCD_IDX)


#include <ZMainWidget.h>

#if HAVE_MODULE(MODULE_MEMINFO)
class MemoryInfo;
#endif
#if HAVE_MODULE(MODULE_MODULES)
class ModulesInfo;
#endif
#if HAVE_MODULE(MODULE_LOAD)
class LoadInfo;
#endif
#if HAVE_MODULE(MODULE_STORAGE)
class FileSysInfo;
#endif
#if HAVE_MODULE(MODULE_PROCESS)
class ProcessInfo;
#endif
#if HAVE_MODULE(MODULE_VERSION)
class VersionInfo;
#endif
#if HAVE_MODULE(MODULE_SYSLOG)
class SyslogInfo;
#endif
#if HAVE_MODULE(MODULE_USB)
class USBMode;
#endif
#if HAVE_MODULE(MODULE_THEME)
class ThemeInfo;
#endif
#if HAVE_MODULE(MODULE_LCD)
class LcdInfo;
#endif
#if HAVE_MODULE(MODULE_BENCHMARK)
class BenchmarkInfo;
#endif

class UTIL_CST;
class QVBox;
class QTimer;
class UTIL_PushButton;

class SystemInfo : public ZMainWidget
{
    Q_OBJECT
public:
  SystemInfo( QWidget *parent = 0, const char *name = 0, unsigned int WFlags= 0 );
 	virtual ~ SystemInfo();

  QString changeUrisdiction( const QString &, const QString &);
public slots:
	void resizeWindow();
	void showFull();
	void about();
	void enterWhatsThisMode();
	void whatsThisTimeout();
	void selectMenu(int);
protected:
	virtual void keyPressEvent ( QKeyEvent * event );
private:
  UTIL_CST *cst;
  QVBox* cw;
  QTimer *wtTimer;
  UTIL_PushButton *wtButton;
  ZPushButton *midBtn, *leftBtn, *resizeButton;;
  QPopupMenu *mPopup, *lPopup;
private:
#if HAVE_MODULE(MODULE_MEMINFO)
	MemoryInfo* memory;
#endif
#if HAVE_MODULE(MODULE_MODULES)
	ModulesInfo* modulesinfo;
#endif
#if HAVE_MODULE(MODULE_LOAD)
	LoadInfo* loadinfo;
#endif
#if HAVE_MODULE(MODULE_STORAGE)
	FileSysInfo* filesysinfo;
#endif
#if HAVE_MODULE(MODULE_PROCESS)
	ProcessInfo* processinfo;
#endif
#if HAVE_MODULE(MODULE_VERSION)
	VersionInfo* versioninfo;
#endif
#if HAVE_MODULE(MODULE_SYSLOG)
	SyslogInfo* sysloginfo;
#endif
#if HAVE_MODULE(MODULE_USB)
	USBMode* usbmode;
#endif
#if HAVE_MODULE(MODULE_THEME)
	ThemeInfo* themeinfo;
#endif
#if HAVE_MODULE(MODULE_LCD)
	LcdInfo* lcdinfo;
#endif
#if HAVE_MODULE(MODULE_BENCHMARK)
	BenchmarkInfo* benchmarkinfo;
#endif
};

#endif //SYSINFO
