/*************************************************************************
 * Theme Manage: List, install, remove, preview and apply themes 
 * Copyright (C) 2009 Meng Zhong Die Wu <497699123@qq.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***********************************************************************/
/*
 Name : THEMES
 By : Meng Zhong Die Wu
 Compile : MOTO-E6-IDE
 Time : 2009.07.26
 EMail : 497699123@qq.com
 
 nukin added: apply the theme immediately
 to do:Logo,Theme editor
*/
#include "themeinfo.h"
#if HAVE_MODULE(MODULE_THEME)
#include "installdialog.h"
#include "picture.h"
#include "filebrowser.h"

#include <AmGlobal.h>
#include <ZPushButton.h>
#include <ZMessageBox.h>
#include <ZSetManager.h>
#include <ZSkin.h>
#include <qdir.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qfont.h>
#include <qpopupmenu.h>
#include <qwhatsthis.h>
#include <ezxutillistbox.h>
#include <ezxutilpushbutton.h>
#include <stdio.h>
#include <stdlib.h>
#include <ezxres.h>
#include <qvbox.h>
#include <qhbox.h>


class ZSkinService
{
public:
	QString colorToString(QColor);
	int getCurrentSkin(QString &);
	int getDefaultSkin(QString &);
	QStringList getSkinList(void);
	void setCurrentSkin(QString);
	QColor stringToColor(QString);
	void updateTextSize(int);
	void updateThemeSkin(QString);
};

class Theme_ListBoxItem : public UTIL_ListBoxItem
{
public:
	Theme_ListBoxItem(QListBox *parent, const QString &f , const QString &t, const QPixmap &pix):UTIL_ListBoxItem(parent, QString::null, FALSE)
	{
		filename = f;
		themesname = t;
		pixmap = pix;
	}
	~Theme_ListBoxItem(){};
	const QString fileName() { return filename ; };
	const QString themeName() { return themesname ; };
protected:
	virtual void paint(QPainter *p)
	{
		p->drawPixmap(1, 0, pixmap );
		p->drawText( 65, 30, themesname);
	}
	int height(QListBox const*) const { return 80; };
	 //int width(QListBox const*) const { return 60; };
private:
	QString filename;
	QString themesname;
	QPixmap pixmap;
};
	
ThemeInfo::ThemeInfo( QWidget *parent, const char *name, WFlags f )
  :QWidget( parent, name, f )
{
	QGridLayout *layout = new QGridLayout( this );
 	layout->setSpacing( 1 );
	layout->setMargin( 5 );

	themeList = new UTIL_ListBox("%T4%T300", this, 0);
	themeList->setFrameStyle(QFrame::NoFrame);
	updateThemeList();
	connect( themeList, SIGNAL( rightButtonPressed(QListBoxItem*, const QPoint&)),this, SLOT( themeListPressed( QListBoxItem*, const QPoint&)) );
	layout->addMultiCellWidget( themeList, 0, 0, 0, 1 );

	QWhatsThis::add( themeList, tr("Theme manage\n read .pskn file as default."));
	checkThemeList();
}

ThemeInfo::~ThemeInfo()
{
}

void ThemeInfo::themeListPressed( QListBoxItem* item, const QPoint& point)
{
	qDebug("menu");
    QPopupMenu  m;
    m.setFont( font() );
    m.insertItem( tr("Install"), this, SLOT( InstallTheme() ));
	if(item!=NULL){
    m.insertItem( tr("Uninstall"), this, SLOT( deleteTheme() ));
    m.insertItem( tr("Apply"), this, SLOT( UseTheme() ));
    m.insertSeparator();
    m.insertItem( tr("PreView"), this, SLOT( slotImageLogo() ));
    m.insertItem( tr("Background"), this, SLOT( slotImageWallpaper() ));
    m.insertItem( tr("Power-on gif"), this, SLOT( slotMovieUp() ));
    m.insertItem( tr("Power-off gif"), this, SLOT( slotMovieDown() ));
    m.insertSeparator();
    m.insertItem( tr("Set Background"), this, SLOT( slotSetWallpaper() ));
    m.insertItem( tr("Set Power-on gif"), this, SLOT( slotSetMovieUp() ));
    m.insertItem( tr("Set Power-off gif"), this, SLOT( slotSetMovieDown() ));
	}
    m.exec( QCursor::pos() );
}

void ThemeInfo::InstallTheme()
{
	QString filename ;
	QString dirname ;
	
	InstallDialog *ID = new InstallDialog( tr("Install"),this);
	ID->setDirName( "/ezxlocal/download/mystuff/Others");
	if( ID->exec() == QDialog::Accepted)
	{
		filename = ID->getFileName();
		filename = filename.utf8();
		dirname = ID->getDirName();
		dirname = dirname.utf8();  //neccesary?
	}else return ;
	if(filename.isEmpty()  ||  dirname.isEmpty() ){
		ZMessageBox::timerMessage(this,NULL,tr("None selected!"),1);
		return;
	}
	
	QDir tmp;
	tmp.mkdir( dirname +"/tmp");
	
	system("/bin/tar xzf " + filename + " -C " + dirname + "/tmp");
	
	QString dir;
	QDir dirtmp(dirname + "/tmp");
	QStringList dir_list = dirtmp.entryList( QDir::NoSymLinks | QDir::Dirs );
	for(int i = 2; i < dir_list.count(); i++){
		dir = Find(dir_list[i]);
	}
	
	if( QDir( dirname+"/"+dir ).exists()){
		ZMessageBox::timerMessage(this,NULL,tr("The theme name exists!"),1);
		return;
	}
	system("/bin/mv "+ dirname +"/tmp/"+dir +" "+dirname+"/"+dir );
	system("/bin/rm -f -r "+dirname+"/tmp");
	
	QString dirpath = dirname + "/" + dir ;  
	QDir skindir( dirpath , "*.pskn" );
	QStringList list = skindir.entryList( QDir::Files );
	for(int i = 0; i < list.count(); i++){
		if ( QFile::exists( dirpath +"/"+ list[i] ) ){
			ReadConfig( dirpath +"/"+ list[i] );
			qDebug( dirpath +"/"+ list[i] );
			
			IconDir = dirpath + "/";
			SkinFile = dirpath + "/" + Find(SkinFile);
			qDebug(SkinFile);
			CurrentUse = dirpath + "/" + Find(CurrentUse);
			LogoIcon = dirpath + "/" + Find(LogoIcon);
			WallPaper = dirpath + "/" + Find(WallPaper);
			PowerOnScreen = dirpath + "/" + Find(PowerOnScreen);
			PowerOffScreen = dirpath + "/" + Find(PowerOffScreen);
			
			SaveConfig( dirpath +"/"+ list[i] );
			system("/bin/cp -r " + dirpath +"/"+ list[i] + " /ezx_user/download/appwrite/skin/" + list[i] );
			
			updateThemeList();
			ZMessageBox::timerMessage(this,NULL,tr("Theme has been installed"),1);
		}
	}
}

void ThemeInfo::deleteTheme()
{
    QListBoxItem *item = themeList->item( themeList->currentItem() );
    QString fn = ( (Theme_ListBoxItem*)(item) )->fileName();
    //qDebug(fn);
    QString CurrentUse_1,IconDir_1;
    SETUP_SET_Manager mgr( fn, false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeCurrentUse", CurrentUse_1 ,"", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeIconDir", IconDir_1 ,"", false);
    if(CurrentUse_1.isEmpty()  ||  IconDir_1.isEmpty() ){
		return;
    }
	
	   RES_ICON_Reader ir;
	   switch (ZMessageBox::information(this, ir.getIcon(RES_ICON_DLG_WAITING,1),tr("Really want to remove this theme?") ,tr("Yes"), tr("NO")))
	   {
	   case 0 : break;
	   default : return;
		   break;
	   }
	   system("/bin/rm -rf " + IconDir_1);
	   system( "/bin/rm -rf " + fn );
	   updateThemeList();
	   ZMessageBox::timerMessage(this,NULL,QString::fromUtf8("主题卸载成功!"),1);
}

void ThemeInfo::UseTheme()
{
	QListBoxItem *item = themeList->item( themeList->currentItem() );
	QString fn = ( (Theme_ListBoxItem*)(item) )->fileName();
	ReadConfig( fn );
	RES_ICON_Reader ir;
	switch (ZMessageBox::information(this, ir.getIcon(RES_ICON_DLG_WAITING,1),tr("Really want to enable this theme?"),tr("Yes"), tr("NO")))
	{
		case 0 : break;
		default : return;
		break;
	}
	SaveConfig("/tmp/ezx_skin.cfg");
	SaveConfig("/ezx_user/download/appwrite/setup/ezx_skin.cfg");
	SaveConfig("/ezx_user/download/appwrite/setup/ezx_theme.cfg");
	//ZSkin::reloadSkin();
	AM_Global::setWallPaper(WallPaper);
	ZSkinService ss;
	ss.updateThemeSkin(CurrentUse);
}

void ThemeInfo::checkThemeList()
{
    QString zt = getThemeName( "/ezx_user/download/appwrite/setup/ezx_skin.cfg" );
    int count = themeList->count();
    for ( int i=0; i<count; i++ )
    {
		QListBoxItem *item = themeList->item(i);
		QFont fnt( ( (Theme_ListBoxItem*)(item) )->font() );
		QString tn = ( (Theme_ListBoxItem*)(item) )->themeName();
		fnt.setBold( tn == zt );
		themeList->checkItem( (Theme_ListBoxItem*)(item),tn == zt);
		((Theme_ListBoxItem*)(item))->setFont( fnt );
    }
}

void ThemeInfo::updateThemeList()
{
    QString fn = "/ezx_user/download/appwrite/skin";
    QDir dir( fn );
    if( ! dir.exists())
	{
		dir.mkdir("/ezx_user/download/appwrite/skin");
		::system("ln -sf /usr/data_resource/skin/* /ezx_user/download/appwrite/skin/");
    }
	
    QDir skindir( fn, "*.pskn" );
    QStringList list = skindir.entryList( QDir::Files );
    int count = list.count();
	
    QString zt = getThemeName( "/ezx_user/download/appwrite/setup/ezx_skin.cfg" );
    for ( int i=0; i<count; i++ )
    {
		QString tn = getThemeName( fn + "/" + list[i] );
		QPixmap pixmap = getThemeLogo( fn + "/" + list[i] );
		Theme_ListBoxItem *item = new Theme_ListBoxItem( themeList, fn + "/" + list[i], tn ,pixmap );
		QFont fnt( item->font() );
		item->setString( 1, tn );
		themeList->insertItem( item );
		fnt.setBold( tn == zt );
		fnt.setUnderline( tn == zt );
		themeList->checkItem( (Theme_ListBoxItem*)(item), tn == zt );
		//themeList->setCurrentItem(0);
		((Theme_ListBoxItem*)(item))->setFont( fnt );
	}
	
	
}

QString ThemeInfo::Find(const QString &path)
{
	QString qstr;
	QStringList linesc = QStringList::split( "/", path );
    for ( int i = 0; i < linesc.count(); i++ ){
		if (i==linesc.count()-1){
			qstr=linesc[i];
		}
	}
	return qstr;
}

const QPixmap ThemeInfo::getThemeLogo(const QString &fileName) 
{
  QString file;
  QPixmap logoPixmap;

  SETUP_SET_Manager mgr( fileName, false);
  mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeLogoIcon", file ,"", false);
  if( file.isEmpty()) return QString::null;
  QImage logo = QImage( file );
  logo = logo.smoothScale( 60, 80 );
  logoPixmap.convertFromImage( logo );
  return logoPixmap;
}

const QString ThemeInfo::getThemeName(const QString &fileName) 
{
  QString file;
  SETUP_SET_Manager mgr( fileName, false);
  mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeName", file ,"", false);
  if( file.isEmpty()) return QString::null;
  return file;
}

void ThemeInfo::slotMovieUp()
{
    moviePreview( TRUE );
}

void ThemeInfo::slotMovieDown()
{
    moviePreview( FALSE );
}

void ThemeInfo::slotImageLogo()
{
    imagePreview( TRUE );
}

void ThemeInfo::slotImageWallpaper()
{
    imagePreview( FALSE );
}

void ThemeInfo::moviePreview( bool on )
{
    QListBoxItem *item = themeList->item( themeList->currentItem() );
    QString fn = ( (Theme_ListBoxItem*)(item) )->fileName();
    //qDebug(fn);
    SETUP_SET_Manager mgr( fn, false);

    if ( on ) {
      mgr.getSettingInfo("THEME_CONFIG_TABLE", "PowerOnScreen", fn ,"/usr/language/animation/Moto1.gif", false);
      fn = fn.stripWhiteSpace();
      if ( !QFile::exists(fn) )
        fn = "/usr/data_resource/animation/Moto1.gif";
    } else {
      mgr.getSettingInfo("THEME_CONFIG_TABLE", "PowerOffScreen", fn ,"/usr/language/animation/Moto2.gif", false);
      fn = fn.stripWhiteSpace();
      if ( !QFile::exists(fn) )
        fn = "/usr/data_resource/animation/Moto2.gif";
    }

    Picture* picture = new Picture(0);
    picture->setPicture(fn);
    picture->show();
}

void ThemeInfo::imagePreview( bool logo )
{
    QListBoxItem *item = themeList->item( themeList->currentItem() );
    QString fn = ( (Theme_ListBoxItem*)(item) )->fileName();

    SETUP_SET_Manager mgr( fn, false);

    if ( logo ) {
      mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeLogoIcon", fn ,"/usr/data_resource/skin/default3/MACAU_03.jpg", false);
      fn = fn.stripWhiteSpace();
      if ( !QFile::exists(fn) )
        fn = "/usr/data_resource/skin/default3/MACAU_03.jpg";
    } else {
      mgr.getSettingInfo("THEME_CONFIG_TABLE", "WallPaper", fn ,"/usr/data_resource/picture/Home_Wallpaper1.gif", false);
      fn = fn.stripWhiteSpace();
      if ( !QFile::exists(fn) )
        fn = "/usr/data_resource/picture/Home_Wallpaper1.gif";
    }

    Picture* picture = new Picture(0);
    picture->setPicture(fn);
    picture->show();
}

void ThemeInfo::slotSetMovieUp()
{
    QString filename ;
    UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
   Dialog->setHasTitle(true);
   Dialog->setDlgTitle("select a gif");

   QVBox *cw = new QVBox(Dialog);
   Dialog->setDlgContent(cw);
   QVBox *vb = (QVBox*)Dialog->getDlgContent();

   FileBrowser *mBrowser = new FileBrowser("*.gif", vb, "browser",1);
   mBrowser->setDir("/mmc/mmca1/");
   QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

   QHBox *hb = new QHBox(Dialog);
   Dialog->setDlgCst(hb);
	
   ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
   okBtn->setText("OK");	
   connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
   ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
   cancelBtn->setText("Cancel");	
   connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));

    if(Dialog->exec() == QDialog::Accepted)
	{
		filename = mBrowser->getFileName();
		setIcon( "/ezx_user/download/appwrite/setup/ezx_skin.cfg","PowerOnScreen", filename );
		setIcon( "/ezx_user/download/appwrite/setup/ezx_theme.cfg","PowerOnScreen", filename );
	}
	delete Dialog;
}

void ThemeInfo::slotSetMovieDown()
{
    QString filename ;
    UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
   Dialog->setHasTitle(true);
   Dialog->setDlgTitle("select a gif");

   QVBox *cw = new QVBox(Dialog);
   Dialog->setDlgContent(cw);
   QVBox *vb = (QVBox*)Dialog->getDlgContent();

   FileBrowser *mBrowser = new FileBrowser("*.gif", vb, "browser",1);
   mBrowser->setDir("/mmc/mmca1/");
   QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

   QHBox *hb = new QHBox(Dialog);
   Dialog->setDlgCst(hb);
	
   ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
   okBtn->setText("OK");	
   connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
   ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
   cancelBtn->setText("Cancel");	
   connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));
    if(Dialog->exec() == QDialog::Accepted)
	   {
       filename = mBrowser->getFileName();
       setIcon( "/ezx_user/download/appwrite/setup/ezx_skin.cfg","PowerOffScreen", filename );
       setIcon( "/ezx_user/download/appwrite/setup/ezx_theme.cfg","PowerOffScreen", filename );
		   }
	  	delete Dialog;
}

void ThemeInfo::slotSetLogo()
{
    QString filename ;
    UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
   Dialog->setHasTitle(true);
   Dialog->setDlgTitle("select a mtf file");

   QVBox *cw = new QVBox(Dialog);
   Dialog->setDlgContent(cw);
   QVBox *vb = (QVBox*)Dialog->getDlgContent();

   FileBrowser *mBrowser = new FileBrowser("*", vb, "browser",1);
   mBrowser->setDir("/mmc/mmca1/");
   QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

   QHBox *hb = new QHBox(Dialog);
   Dialog->setDlgCst(hb);
	
   ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
   okBtn->setText("OK");	
   connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
   ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
   cancelBtn->setText("Cancel");	
   connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));
    if(Dialog->exec() == QDialog::Accepted)
	   {
       filename = mBrowser->getFileName();
       setIcon( "/ezx_user/download/appwrite/setup/ezx_skin.cfg","ThemeLogoIcon", filename  );
       setIcon( "/ezx_user/download/appwrite/setup/ezx_theme.cfg","ThemeLogoIcon", filename );
		   }
	  	delete Dialog;
}

void ThemeInfo::slotSetWallpaper()
{
    QString filename ;
    UTIL_Dialog *Dialog = new UTIL_Dialog(UTIL_Dialog::DTLargeSize,true, this,"OpenFile Dialog",1,0);
	Dialog->setHasTitle(true);
	Dialog->setDlgTitle("select a mtf file");

	QVBox *cw = new QVBox(Dialog);
	Dialog->setDlgContent(cw);
	QVBox *vb = (QVBox*)Dialog->getDlgContent();

	FileBrowser *mBrowser = new FileBrowser("*", vb, "browser",1);
	mBrowser->setDir("/mmc/mmca1/");
	QObject::connect(mBrowser, SIGNAL(isFilePicked(bool)), SLOT(filePicked(bool)));

	QHBox *hb = new QHBox(Dialog);
	Dialog->setDlgCst(hb);
	
	ZPushButton *okBtn = new ZPushButton(hb, 0, -1, -1);
	okBtn->setText("OK");	
	connect( okBtn, SIGNAL( clicked() ), Dialog, SLOT(accept()));
	
	ZPushButton *cancelBtn = new ZPushButton(hb, 0, -1, -1);
	cancelBtn->setText("Cancel");	
	connect( cancelBtn, SIGNAL( clicked() ), Dialog, SLOT(reject()));
	if(Dialog->exec() == QDialog::Accepted)
	{
		filename = mBrowser->getFileName();
		setIcon( "/ezx_user/download/appwrite/setup/ezx_skin.cfg","WallPaper", filename  );
		setIcon( "/ezx_user/download/appwrite/setup/ezx_theme.cfg","WallPaper", filename );
		AM_Global::setWallPaper( filename );  //SETUP_Utility::setAsWallpaper(QString)
	}
	delete Dialog;
}

void ThemeInfo::setIcon(const QString &filename ,const QString &type , const QString &var)
{
    SETUP_SET_Manager mgr( filename , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", type , var , false);
}

void ThemeInfo::ReadConfig( const QString &filename)
{
    SETUP_SET_Manager mgr( filename , false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "BlendingColor", BlendingColor ,"0 0 0", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "BlendingPercentage", BlendingPercentage , 85);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeLogoIcon", LogoIcon ,"/usr/data_resource/skin/default3/MACAU_03.jpg", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeSkinFile", SkinFile ," /usr/data_resource/skin/default3/MACAU_03.skn", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeIconDir", IconDir ," /usr/data_resource/skin/default3", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeTextSize", TextSize , 0);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeColorPaletteIndex", ColorPaletteIndex , 3);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeName", Name ,"Maldives Blue", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "ThemeCurrentUse", CurrentUse ,"MACAU_01.pskn", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "WallPaper", WallPaper,"/usr/data_resource/picture/Home_Wallpaper.jpg", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "PowerOnScreen", PowerOnScreen ,"/usr/data_resource/animation/Moto1.gif", false);
    mgr.getSettingInfo("THEME_CONFIG_TABLE", "PowerOffScreen", PowerOffScreen ,"/usr/data_resource/animation/Moto2.gif", false);
}

void ThemeInfo::SaveConfig( const QString &filename )
{
    SETUP_SET_Manager mgr( filename , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "BlendingColor", BlendingColor , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "BlendingPercentage", BlendingPercentage );
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeLogoIcon", LogoIcon , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeSkinFile", SkinFile , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeIconDir", IconDir , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeTextSize", TextSize);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeColorPaletteIndex", ColorPaletteIndex);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeName", Name , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "ThemeCurrentUse", CurrentUse , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "WallPaper", WallPaper , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "PowerOnScreen", PowerOnScreen , false);
    mgr.setSettingInfo("THEME_CONFIG_TABLE", "PowerOffScreen", PowerOffScreen , false);
}
#endif //HAVE_MODULE(MODULE_THEME)
