/*
 * USBMode for EZX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 Name : USB MODE
 By : Meng Zhong Die Wu
 Compile : MOTO-E6-IDE
 Time : 2009.07.26
 EMail : 497699123@qq.com
*/
#include "usbmode.h"

#include <ZSetManager.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qfile.h>
#include <qwhatsthis.h>
#include <qtextstream.h>
#include <ezxutillistbox.h>

#include <stdio.h>

USBMode::USBMode( QWidget* parent, const char* name, WFlags fl)
	: QWidget( parent, name, fl )
{
  QVBoxLayout *layout = new QVBoxLayout( this, 0, 0 );
  	
  ListBox = new UTIL_ListBox( "%T4%R20%T300", this, 0);              //???????????????/
  ListBox->setFrameStyle(QFrame::NoFrame);
  ListBox->setMargin( 0 );
  ListBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
  connect(ListBox, SIGNAL(clicked(QListBoxItem*)), SLOT(userChangeState(QListBoxItem*)));

  QWhatsThis::add( ListBox, tr( "This is a list of all the USB mode supported on this handheld device.\n\nClick on a mode to define which mode you want to activate." ) );

  layout->addWidget(ListBox);
  
  StringList.append( tr("Modem") );
  StringList.append( tr("Mass Storage") );
  StringList.append( tr("Product Support"));
  StringList.append( tr("USB Net") );
  StringList.append( tr("Web Camera") );
  
  String[0] = "MotACM";
  String[1] = "MotMas";
  String[2] = "MotPst";
  String[3] = "MotNet";
  String[4] = "MotCdc";

  //USB listbox
  for(QStringList::ConstIterator it = StringList.begin();it != StringList.end(); it++){
    ListBoxItem = new UTIL_ListBoxItem(ListBox, QString::null, false);
    ListBoxItem->setString(2, *it); 
    ListBox->insertItem(ListBoxItem);
  }

  QFrame *f = new QFrame( this );
	 layout->addWidget(f);
	 f->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  Title = new QLabel("USB Status",this);
  Title->setMargin( 4 );
  layout->addWidget(Title);
  
  QWhatsThis::add( Title, tr( "This is additional information about the currently active USB mode." ) );
  
  startTimer(2000);  
}
USBMode::~USBMode()
{
}
void USBMode::timerEvent(QTimerEvent*)
{
  refreshState();
}
void USBMode::userChangeState(QListBoxItem* item)
{
  writeToProcFile(String[item->listBox()->currentItem()]);	
}
void USBMode::refreshState()
{
	QString status;
	char buf[128];
	int i, l, cable, charging;
  
	FILE *fd = fopen("/proc/usbd-switch", "r");
  
	if (fd == NULL) status.sprintf("Could not read status!");
	else if(fgets(buf, sizeof(buf) - 1, fd))
	{
		l = strlen(buf);
		while (l && (buf[l-1] == '\n')) { l--; buf[l] = '\0'; }
		for (i = 0; i< StringList.count(); i++)
		{
			if ( String[i] == QString(buf)) 
			{
				ListBox->setCurrentItem(i);
				break;
			}
		}
		if ( i < StringList.count()) 
		{
			status = String[i];
			status.append('\n');
		}
		else status.sprintf("Unknown mode: \"%s\"\n", buf);

		if (fscanf(fd, "IP=%s\nCABLE=%d\nCHARGING=%d", buf, &cable , &charging) == 3)
		{
			if (!cable) status.append( tr("Not connected"));
			else if(!charging) status.append(tr("Connected"));
			else
			{
				QString md;
				md.sprintf( tr("Connected, Charging: %dmA"), charging);
				status.append(md);
			}
			if( String[i] == "MotNet" )
			{
				QString ipa;
				ipa.sprintf("\nIP: %s", buf);
				status.append(ipa);
			}
		}
		Title->setText(status);
	}
	if(fd) fclose(fd);	
}

void USBMode::writeToProcFile(const QString& text)
{
	int var = ListBox->currentItem();
	if(var > 2) var = var-1;
	SETUP_SET_Manager mgr( "/ezx_user/download/appwrite/setup/ezx_system.cfg", false);
	mgr.setSettingInfo("SYS_IRDA_BT", "USBStatus", var);

	QFile file("/proc/usbd-switch");
	if ( file.open( IO_WriteOnly ) )
	{
		QTextStream stream( &file );
		stream << text;
    //qDebug(text);
	}
	file.close();
}
/*void USBModeWidget::writeToProcFile(const char *text)
{
	FILE *fd;
	fd = fopen("/proc/usbd-switch", "w+");
	if (fd != NULL) {
		fputs(text, fd);
		fclose(fd);
	}
}*/
