/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "versioninfo.h"
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qscrollview.h>
#include <qtextstream.h>
//#include <qtextview.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <stdlib.h>

VersionInfo::VersionInfo( QWidget *parent, const char *name, WFlags f )
    : QWidget( parent, name, f )
{
    setMinimumSize( 200, 150 );

    QFont fnt( font() );
    fnt.setPointSize( 12 );
    setFont( fnt );

    QVBoxLayout *tmpvb = new QVBoxLayout( this );
    QScrollView *sv = new QScrollView( this );
    tmpvb->addWidget( sv, 0, 0 );
    //sv->setHScrollBarMode(QScrollView::AlwaysOff);
    sv->setResizePolicy( QScrollView::AutoOneFit );
    //sv->setHScrollBarMode( QScrollView::AlwaysOff );
    //sv->setVScrollBarMode( QScrollView::AlwaysOff );
    sv->setFrameStyle( QFrame::NoFrame );
    QWidget *container = new QWidget( sv->viewport() );
    sv->addChild( container );

    QVBoxLayout *vb = new QVBoxLayout( container, 3 );

    //About Title
    /*QHBoxLayout *About = new QHBoxLayout( vb );
    About->setSpacing( 1 );
    QLabel *AboutLogo = new QLabel( container );
    QImage logo = QImage( "icon-big.png" );
    logo = logo.smoothScale( 40, 40 );
    QPixmap logoPixmap;
    logoPixmap.convertFromImage( logo );
    AboutLogo->setPixmap( logoPixmap );
    AboutLogo->setFixedSize( 40, 40 );
    About->addWidget( AboutLogo, 0, Qt::AlignTop + Qt::AlignLeft );
    QString AboutString = "<qt><b>";
    AboutString.append( QString::fromUtf8("任务管理 V:1.0.0") );
    AboutString.append( "</b>" );
    AboutString.append(QString::fromUtf8("<br>说明 : 这个程序是自由软件,禁止用于商业目地!"));
    AboutString.append(QString::fromUtf8("<br>适用 : MOTO A1200/E6"));
    AboutString.append(QString::fromUtf8("<br>制作 : 移动天使/梦中蝶舞"));
    AboutString.append(QString::fromUtf8("<br>邮箱 : 497699123@qq.com"));
    AboutString.append(QString::fromUtf8("<br>网址 : http://bbs.512530.com/forum-3475-1.html"));
    AboutString.append(QString::fromUtf8("<br>编译 : MOTO-E6-IDE"));
    AboutString.append(QString::fromUtf8("<br>时间 : 2009.07.22"));
    AboutString.append("</qt>");
    QLabel *AboutVersion = new QLabel( container );
    AboutVersion->setText( AboutString );
    About->addWidget( AboutVersion, 1, Qt::AlignTop + Qt::AlignLeft );*/

    //system version
    QHBoxLayout *hb0 = new QHBoxLayout( vb );
    hb0->setSpacing( 2 );
    QLabel *motorolaLogo = new QLabel( container );
    QImage logo0 = QImage( "pics/motorola-logo.png" );
    logo0 = logo0.smoothScale( 40, 40 );
    QPixmap logo0Pixmap;
    logo0Pixmap.convertFromImage( logo0 );
    motorolaLogo->setPixmap( logo0Pixmap );
    motorolaLogo->setFixedSize( 40, 40 );
    hb0->addWidget( motorolaLogo, 0, Qt::AlignTop + Qt::AlignLeft );
    QString firmwareString = "<qt><b>";
    firmwareString.append( tr("Firmware") );
    firmwareString.append( "</b>" );

    if ( QFile::exists( "/etc/ap_version.txt" ) )
    {
        QFile ap( "/etc/ap_version.txt" );
        if ( ap.open( IO_ReadOnly ) )
        {
            QTextStream as( &ap );
            firmwareString.append( tr( "<p>AP: " ) );
            firmwareString.append( as.read() );
            ap.close();
        }
    }

    if ( QFile::exists( "/etc/ap_version.txt" ) )
    {
        QFile bp( "/tmp/VersionInfo" );
        if ( bp.open( IO_ReadOnly ) )
        {
            QTextStream bs( &bp );
            firmwareString.append( tr( "<br>BP: " ) );
            firmwareString.append( QString(bs.readLine()).mid(0,19) );
            //firmwareString.append( tr( "<br>BPFlex: " ) );
            //firmwareString.append( QString(bs.readLine()).mid(48,69) );
           // firmwareString.append( tr( "<br>DSP: " ) );
           // firmwareString.append( QString(bs.readLine()).mid(96,104) );
            //firmwareString.append( tr( "<br>Technology: " ) );
            //firmwareString.append( QString(bs.readLine()).mid(144,156) );
            bp.close();
        }
    }

    firmwareString.append("</qt>");
    QLabel *firmwareVersion = new QLabel( container );
    firmwareVersion->setText( firmwareString );
    hb0->addWidget( firmwareVersion, 1, Qt::AlignTop + Qt::AlignLeft );
    // QT/E Version
    QHBoxLayout *hb1 = new QHBoxLayout( vb );
    hb1->setSpacing( 2 );
    QLabel *palmtopLogo = new QLabel( container );
    // QImage logo1 = Resource::loadImage( "logo/opielogo" );
    QImage logo1 = QImage( "pics/qtlogo.png" );
    logo1 = logo1.smoothScale( 40, 40 );
    QPixmap logo1Pixmap;
    logo1Pixmap.convertFromImage( logo1 );
    palmtopLogo->setPixmap( logo1Pixmap );
    palmtopLogo->setFixedSize( 40, 40 );
    hb1->addWidget( palmtopLogo, 0, Qt::AlignTop + Qt::AlignLeft );

    QString palmtopVersionString = "<qt>" + tr( "<b>Qt/E</b><p>Version : " );
    // palmtopVersionString.append( tr( "Built against Qt/E " ) );
    palmtopVersionString.append( QT_VERSION_STR );
	palmtopVersionString.append( "<br>" );
	palmtopVersionString.append( tr( "Built on: " ) );
	palmtopVersionString.append( __DATE__  );
    palmtopVersionString.append( "</qt>" );

    QLabel *palmtopVersion = new QLabel( container );
    palmtopVersion->setText( palmtopVersionString  );
    hb1->addWidget( palmtopVersion, 1, Qt::AlignTop + Qt::AlignLeft );

    // kernel Version
    QHBoxLayout *hb2 = new QHBoxLayout( vb );
    hb1->setSpacing( 2 );
    QLabel *linuxLogo = new QLabel( container );
    QImage logo2 = QImage( "pics/tux-logo.png" );
    logo2 = logo2.smoothScale( 40, 40 );
    QPixmap logo2Pixmap;
    logo2Pixmap.convertFromImage( logo2 );
    linuxLogo->setPixmap( logo2Pixmap );
    linuxLogo->setFixedSize( 40, 40 );
    hb2->addWidget( linuxLogo, 0, Qt::AlignTop + Qt::AlignLeft );

    QString kernelVersionString;
    QFile file( "/proc/version" );
    if ( file.open( IO_ReadOnly ) )
    {
        QTextStream t( &file );
        QStringList strList;

        strList = QStringList::split(  " " , t.read(),  false );

        kernelVersionString = "<qt>" + tr( "<b>Linux kernel</b><p>Version: " );
        kernelVersionString.append( strList[2] );
        kernelVersionString.append( "<br>" );
        kernelVersionString.append( tr( "Compiled by: " ) );
        kernelVersionString.append( strList[3] );
        kernelVersionString.append("</qt>");
        file.close();
    }
    QLabel *kernelVersion = new QLabel( container );
    kernelVersion->setText( kernelVersionString );
    hb2->addWidget( kernelVersion, 1, Qt::AlignTop + Qt::AlignLeft );

    //distribution information 
    QHBoxLayout *hb3 = new QHBoxLayout( vb );
    hb3->setSpacing( 2 );
    QLabel *palmtopLogo3 = new QLabel( container );
    QImage logo3 = QImage( "pics/pda.png" );
    logo3 = logo3.smoothScale( 40, 40 );
    QPixmap logo3Pixmap;
    logo3Pixmap.convertFromImage( logo3 );
    palmtopLogo3->setPixmap( logo3Pixmap );
    palmtopLogo3->setFixedSize( 40, 40 );
    hb3->addWidget( palmtopLogo3, 0, Qt::AlignTop + Qt::AlignLeft );

    QString s;
    if ( QFile::exists( "/etc/issue" ) )
    {
        QFile tf( "/etc/issue" );
        if ( tf.open( IO_ReadOnly ) )
        {
            QTextStream ts( &tf );
            s = ts.readLine().replace( QRegExp( "\\\\." ), "" );
            tf.close();
        }
    }

    QString systemString = "<qt><b>";
    systemString.append( tr("OS") );
    systemString.append( "</b>" );
    systemString.append( tr( "<p>Version : " ) );
    systemString.append( s );

    QFile lf( "/proc/cpuinfo" );
    if ( lf.open( IO_ReadOnly ) )
    {
        QTextStream ls( &lf );
        while ( !ls.atEnd() )
        {
            QString line;
            line = ls.readLine();
            if ( line.startsWith( "Hardware" ) )
            {
                s = line;
                break;
            }
        }
        lf.close();
        s = s.replace( 0, 10, "" ); // Hardware :
    }

    systemString.append( tr( "<br>Model: " ) );
    systemString.append( s );
    systemString.append( tr( "<br>Vendor: " ) );
    systemString.append( "Motorola" );
    systemString.append("</qt>");

    QLabel *systemVersion = new QLabel( container );
    systemVersion->setText( systemString );
    hb3->addWidget( systemVersion, 1, Qt::AlignTop + Qt::AlignLeft );

    QWhatsThis::add( this, tr( "This page shows the current versions of firmware, Qt, the Linux kernel and distribution running on this handheld device." ) );
}
VersionInfo::~VersionInfo()
{
}

